package calc;

import java.io.IOException;
import java.util.Random;

import util.DialogUtil;

/*
*  DamageCalcクラス
*
*  ダメージ計算に必要なメソッドを提供するクラスです。
*
*  変更履歴
*  2019/05/03 新規作成
*  2019/05/04 身代わり最終ダメージと回復最終回復量のメソッドを廃止（state系クラスのメソッドで既に同様のロジック有り）
*  2019/05/04 急所率計算メソッドについて、急所技フラグの代わりに技コードで判定する方式に変更
*  2019/05/05 純鋼に対するプレゼント倍率が判明したので倍率を訂正
*  2019/05/06 タイプ相性で0倍になる時にメッセージを出力するように変更
*  2019/05/06 追加効果判定メソッドを追加
*  2019/05/10 計算結果の途中でintの暗黙の型変換が行われていたバグを修正
*  2019/05/10 ABCDランク計算用メソッドを素早さランク計算用にも流用する方針とした
*             （計算速度より正確性を重視するためにBigDecimalの使用を検討したが、逆にPBSの結果と合わなくなったため）
*  2019/05/25 最終ダメージ確定メソッドについて、連続攻撃のバグを訂正（HP0→1防止、気合いのハチマキ判定は１回だけ）
*  2019/05/25 実機では未来予知に気合のハチマキが発動されるので、最終ダメージ確定（現HP・HP1残し考慮）メソッドを分割
*             (PBSでは不明だが実機と同じ仕様であると仮定)
*  2019/05/25 命中率計算メソッドについて、計算結果が負の値になる場合に0にするように修正
*  2019/05/26 被ロックオンの実装漏れを修正
*  2019/08/13 ダメージN倍化計算メソッド（PBSトリプルキックバグ再現用）を追加
*  2019/10/13 純電気に対するプレゼントのダメージ倍率を1/3から1/2に変更
*  2019/11/04 タイプ相性が１倍未満か１倍か１倍超えかを表示するように変更
*  2019/11/30 どく/じめんに対するプレゼントのダメージ倍率を4/21から3/4に変更
*  2019/12/31 プレゼント計算式v0.3版に対応するメソッドを追加
*  2020/08/28 気合のハチマキ発動処理を独自のメソッドとして外出し
*  2020/10/15 攻撃力・防御力のカンスト処理追加
*  2020/10/29 持ち物の「どくバリ」が「どくばり」になっていたのを修正
*  2020/11/24 攻撃力または防御力が255を超える場合は4で割る処理が漏れていたので修正
*  2020/12/19 最終ダメージ確定メソッドの意味のない記述をコメントアウト（何か意図があったのかもしれないので消さない）
*  2021/03/11 「くろいメガネ」表記が残っていたので「くろいめがね」に訂正
*
*/

public class DamageCalc {

	/* 命中率計算メソッド
	 *   引数　　　：技の命中率（double）、
	 *   　　　　　：攻撃側の命中率ランクに＋６した値（0～12）(int)、防御側の回避率ランクに＋６した値（0～12）(int)、
	 *   　　　　　：防御側の持ち物（String）、攻撃側の見破るフラグ（boolean）、防御側の被見破るフラグ（boolean）
	 *   　　　　　：防御側の被ロックオンフラグ（boolean）
	 *   戻り値　　：ランク・光の粉考慮の命中率（double）
	 *   例外　　　：無し
	 */
    public double hitProbabilityCalc(double beforeHitProbability,
    		                           int hitProbabilityRankTmp,int avoidProbabilityRankTmp,
    		                           String itemName,boolean seeingThroughFlag,boolean seenThroughFlag,
    		                           boolean lockonFlag) {
    	int hitProbabilityRank = hitProbabilityRankTmp;
    	int avoidProbabilityRank = avoidProbabilityRankTmp;
    	double afterHitProbability = 100.0;
    	double[] hitProbabilityRankRatio = {3.0/9.0, 3.0/8.0, 3.0/7.0, 3.0/6.0, 3.0/5.0, 3.0/4.0, 3.0/3.0
    			                                ,4.0/3.0, 5.0/3.0, 6.0/3.0, 7.0/3.0, 8.0/3.0, 9.0/3.0};
    	double[] avoidProbabilityRankRatio = {9.0/3.0, 8.0/3.0, 7.0/3.0, 6.0/3.0, 5.0/3.0, 4.0/3.0, 3.0/3.0
    			                                ,3.0/4.0, 3.0/5.0, 3.0/6.0, 3.0/7.0, 3.0/8.0, 3.0/9.0};
    	if (seeingThroughFlag && seenThroughFlag) {
    		if (hitProbabilityRank < avoidProbabilityRank) {
    			hitProbabilityRank = 6;
    			avoidProbabilityRank = 6;
    		}
    	}
    	double lightPowderProbability = 0;
    	if (itemName.equals("ひかりのこな")) {
    		lightPowderProbability = 7.8;
    	}
    	afterHitProbability = beforeHitProbability * hitProbabilityRankRatio[hitProbabilityRank]
                                                   * avoidProbabilityRankRatio[avoidProbabilityRank]
                                                   - lightPowderProbability;
    	if (afterHitProbability < 0.0) {
    		afterHitProbability = 0.0;
    	}
    	if (lockonFlag) {
    		afterHitProbability = 100.0;
    	}
       	return afterHitProbability;
    }

	/* 命中判定メソッド
	 *   引数　　　：ランク・光の粉考慮の命中率（double）、
	 *   　　　　　：防御側の守る成功フラグ（boolean）、防御側の直近技コード（String）
	 *   戻り値　　：命中したか否か（boolean）
	 *   例外　　　：IOException（対話式でログファイルを出力できない場合）
	 */
    public boolean hitJudge(double afterHitProbability,
    		                  boolean protectSuccessFlag,String latestUsedMoveEffectCode) throws IOException {
    	Random random = new Random();
    	int hitRandom = random.nextInt(10000);
    	int hitBoarder = (int) Math.floor(afterHitProbability * 100.0);
    	if (protectSuccessFlag && latestUsedMoveEffectCode.equals("6F")) {
    		hitBoarder = 0;
    	}
    	if (hitRandom < hitBoarder) {
    		return true;
    	} else {
    		DialogUtil.dialogMessageOutput("攻撃は外れた");
    		return false;
    	}
    }

	/* 攻撃力計算メソッド（袋叩き・プレゼント・混乱自滅除く）
	 *   引数　　　：攻撃技のタイプ（String）、ランク変動前の攻撃（double）、ランク変動前の特攻（double）、
	 *   　　　　　：攻撃ランクに＋６した値（0～12）（int）、特攻ランクに＋６した値（0～12）（int）、
	 *   　　　　　：攻撃側の種族名（String）、攻撃側の持ち物（String）、攻撃側の状態異常（String）
	 *   戻り値　　：攻撃力（double）
	 *   例外　　　：無し
	 */
    public double subjectValueCalc(String moveType,double beforeAttack,double beforeContact,
    		                          int attackRank,int contactRank,
    		                          String speciesName,String itemName,String badHealth) {
		double subjectValue;
    	if(moveType.equals("normal")||
    	   moveType.equals("fighting")||
    	   moveType.equals("flying")||
		   moveType.equals("poison")||
		   moveType.equals("ground")||
		   moveType.equals("rock")||
		   moveType.equals("bug")||
		   moveType.equals("ghost")||
		   moveType.equals("steel")) {
    		subjectValue = beforeAttack;
    		subjectValue = ABCDSRankCalc(subjectValue,attackRank);
    		subjectValue = thickBoneCalc(subjectValue,speciesName,itemName);
    		if (subjectValue > 999.0) {
    			subjectValue = 999.0;
    		}
    		subjectValue = burnCalc(subjectValue,badHealth);
    	} else {
    		subjectValue = beforeContact;
    		subjectValue = ABCDSRankCalc(subjectValue,contactRank);
    		subjectValue = electricBallCalc(subjectValue,speciesName,itemName);
    		if (subjectValue > 999.0) {
    			subjectValue = 999.0;
    		}
    	}
    	return subjectValue;
    }

	/* 攻撃力計算メソッド（プレゼント用）　※v0.3版計算式で廃止
	 *   引数　　　：防御側のタイプ１（String）、防御側のタイプ２（String）
	 *   戻り値　　：攻撃力（double）
	 *   例外　　　：無し
	 */
    public double presentSubjectValueCalc(String type1,String type2) {
    	double subjectValue;
    	subjectValue = presentTypeRatioCalc(type1,type2);
		if (subjectValue > 999.0) {
			subjectValue = 999.0;
		}
    	return subjectValue;
    }

	/* 攻撃力計算メソッド（プレゼント用）　※v0.3版
	 *   引数　　　：防御側のタイプ１（String）、防御側のタイプ２（String）、
	 *   　　　　　：攻撃ランクに＋６した値（0～12）（int）、
	 *   　　　　　：攻撃側の種族名（String）、攻撃側の持ち物（String）、攻撃側の状態異常（String）、
	 *   　　　　　：攻撃側の見破るフラグ（boolean）、防御側の被見破るフラグ（boolean）
	 *   戻り値　　：攻撃力（double）
	 *   例外　　　：IOException（対話式でログファイルを出力できない場合）
	 */
    public double presentSubjectValueCalc_v_0_3(String type1,String type2,
                                                  int attackRank,
                                                  String speciesName,String itemName,String badHealth,
                                                  boolean seeingThroughFlag,boolean seenThroughFlag) throws IOException{
    	double subjectValue;
    	subjectValue = 100.0/(double)(200 - typeCompatibilityCalc(100,"normal",type1,type2,seeingThroughFlag,seenThroughFlag));
		subjectValue = ABCDSRankCalc(subjectValue,attackRank);
		subjectValue = thickBoneCalc(subjectValue,speciesName,itemName);
		if (subjectValue > 999.0) {
			subjectValue = 999.0;
		}
		subjectValue = burnCalc(subjectValue,badHealth);
    	return subjectValue;
    }

	/* 攻撃力計算メソッド（混乱自滅用）
	 *   引数　　　：ランク変動前の攻撃（double）、
	 *   　　　　　：攻撃ランクに＋６した値（0～12）（double）、
	 *   　　　　　：攻撃側の状態異常（String）
	 *   戻り値　　：攻撃力（double）
	 *   例外　　　：無し
	 */
    public double confuseSubjectValueCalc(double beforeAttack,int attackRank,String badHealth) {
    	double subjectValue;
    	subjectValue = beforeAttack;
    	subjectValue = ABCDSRankCalc(subjectValue,attackRank);
		if (subjectValue > 999.0) {
			subjectValue = 999.0;
		}
    	subjectValue = burnCalc(subjectValue,badHealth);
    	return subjectValue;
    }

	/* 防御力計算メソッド（袋叩き・プレゼント・混乱自滅除く）
	 *   引数　　　：攻撃技のタイプ（String）、ランク変動前の防御（double）、ランク変動前の特防（double）、
	 *   　　　　　：防御ランクに＋６した値（0～12）（int）、特防ランクに＋６した値（0～12）（int）、
	 *   　　　　　：防御側の種族名（String）、防御側の持ち物（String）、
	 *   　　　　　：防御側の自爆・大爆発選ばれたフラグ（boolean）、
	 *   　　　　　：防御側のリフレクター継続ターン（int）、防御側の光の壁継続ターン（int）
	 *   戻り値　　：攻撃力（double）
	 *   例外　　　：無し
	 */
    public double objectValueCalc(String moveType,double beforeBlock,double beforeDiffense,
                                    int blockRank,int diffenseRank,
                                    String speciesName,String itemName,
                                    boolean blockHalfFlag,
                                    int reflectorRestTurn,int lightWallRestTurn) {
    	double objectValue;
    	if(moveType.equals("normal")||
    	   moveType.equals("fighting")||
    	   moveType.equals("flying")||
    	   moveType.equals("poison")||
    	   moveType.equals("ground")||
    	   moveType.equals("rock")||
    	   moveType.equals("bug")||
    	   moveType.equals("ghost")||
    	   moveType.equals("steel")||
     	   moveType.equals("physic")) {
    		objectValue = beforeBlock;
    		objectValue = ABCDSRankCalc(objectValue,blockRank);
    		objectValue = metalPowderCalc(objectValue,speciesName,itemName);
    		objectValue = reflectorBlockCalc(objectValue,reflectorRestTurn);
    		if (objectValue > 999.0) {
    			objectValue = 999.0;
    		}
    		objectValue = explosionBlockHalfCalc(objectValue,blockHalfFlag);
    	} else {
    		objectValue = beforeDiffense;
    		objectValue = ABCDSRankCalc(objectValue,diffenseRank);
    		objectValue = metalPowderCalc(objectValue,speciesName,itemName);
    		objectValue = lightWallDefenceCalc(objectValue,lightWallRestTurn);
    		if (objectValue > 999.0) {
    			objectValue = 999.0;
    		}
    	}
    	return objectValue;
    }

	/* 防御力計算メソッド（プレゼント用）　※v0.3版計算式で廃止
	 *   引数　　　：攻撃側のタイプ１（String）、攻撃側のタイプ２（String）
	 *   戻り値　　：攻撃力（double）
	 *   例外　　　：無し
	 */
    public double presentObjectValueCalc(String type1,String type2) {
    	double objectValue;
    	objectValue = presentTypeNumberCalc(type1,type2);
		if (objectValue > 999.0) {
			objectValue = 999.0;
		}
    	return objectValue;
    }

	/* 防御力計算メソッド（プレゼント用）　※v0.3版
	 *   引数　　　：攻撃側のタイプ１（String）、攻撃側のタイプ２（String）、
	 *   　　　　　：防御ランクに＋６した値（0～12）（int）、
	 *   　　　　　：防御側の種族名（String）、防御側の持ち物（String）、
	 *   　　　　　：防御側の自爆・大爆発選ばれたフラグ（boolean）、
	 *   　　　　　：防御側のリフレクター継続ターン（int）
	 *   戻り値　　：攻撃力（double）
	 *   例外　　　：無し
	 */
    public double presentObjectValueCalc_v_0_3(String type1,String type2,
                                                 int blockRank,
                                                 String speciesName,String itemName,
                                                 boolean blockHalfFlag,
                                                 int reflectorRestTurn) {
    	double objectValue;
    	objectValue = presentTypeNumberCalc_v_0_3(type1,type2);
		objectValue = ABCDSRankCalc(objectValue,blockRank);
		objectValue = metalPowderCalc(objectValue,speciesName,itemName);
		objectValue = reflectorBlockCalc(objectValue,reflectorRestTurn);
		if (objectValue > 999.0) {
			objectValue = 999.0;
		}
		objectValue = explosionBlockHalfCalc(objectValue,blockHalfFlag);
    	return objectValue;
    }

	/* 防御力計算メソッド（混乱自滅用）
	 *   引数　　　：ランク変動前の防御（double）、
	 *   　　　　　：防御ランクに＋６した値（0～12）（int）、
	 *   　　　　　：防御側の自爆・大爆発選ばれたフラグ（boolean）、
	 *   　　　　　：防御側のリフレクター継続ターン（int）
	 *   戻り値　　：攻撃力（double）
	 *   例外　　　：無し
	 */
    public double confuseSubjectValueCalc(double beforeBlock,int blockRank,
    		                                boolean blockHalfFlag,int reflectorRestTurn) {
    	double objectValue;
		objectValue = beforeBlock;
		objectValue = ABCDSRankCalc(objectValue,blockRank);
		objectValue = reflectorBlockCalc(objectValue,reflectorRestTurn);
		if (objectValue > 999.0) {
			objectValue = 999.0;
		}
		objectValue = explosionBlockHalfCalc(objectValue,blockHalfFlag);
    	return objectValue;
    }

	/* レベル計算メソッド（プレゼント用）　※v0.3版計算式で廃止
	 *   引数　　　：防御側のタイプ１（String）、防御側のタイプ２（String）
	 *   戻り値　　：レベル（double）
	 *   例外　　　：無し
	 */
    public double presentLevelValueCalc(String type1,String type2) {
    	double levelValue;
    	levelValue = presentTypeNumberCalc(type1,type2);
    	return levelValue;
    }

	/* レベル計算メソッド（プレゼント用）　※v0.3版
	 *   引数　　　：防御側のタイプ１（String）、防御側のタイプ２（String）
	 *   戻り値　　：レベル（double）
	 *   例外　　　：無し
	 */
    public double presentLevelValueCalc_v_0_3(String type1,String type2) {
    	double levelValue;
    	levelValue = presentTypeNumberCalc_v_0_3(type1,type2);
    	return levelValue;
    }

	/* ABCDSランク計算メソッド
	 *   引数　　　：ランク変動前のステータス（double）、ランクに＋６した値（0～12）（int）
	 *   戻り値　　：ランク変動後のステータス（double）
	 *   例外　　　：無し
	 */
    public double ABCDSRankCalc(double beforeStatus,int rank) {
    	double[] rankRatio = {2.0/8.0, 2.0/7.0, 2.0/6.0, 2.0/5.0, 2.0/4.0, 2.0/3.0, 2.0/2.0
    			                ,3.0/2.0, 4.0/2.0, 5.0/2.0, 6.0/2.0, 7.0/2.0, 8.0/2.0};
    	return Math.floor(beforeStatus * rankRatio[rank]);
    }

	/* 太いホネ計算メソッド
	 *   引数　　　：太いホネ考慮前の攻撃（double）、攻撃側の種族名（String）、攻撃側の持ち物（String）
	 *   戻り値　　：太いホネ考慮後の攻撃（double）
	 *   例外　　　：無し
	 */
    public double thickBoneCalc(double beforeAttack,String speciesName,String itemName) {
    	if ((speciesName.equals("カラカラ") || speciesName.equals("ガラガラ")) &&
    		 itemName.equals("ふといホネ")) {
    		return beforeAttack * 2.0;
    	} else {
    		return beforeAttack;
    	}
    }

	/* 電気玉計算メソッド
	 *   引数　　　：電気玉考慮前の特攻（double）、攻撃側の種族名（String）、攻撃側の持ち物（String）
	 *   戻り値　　：電気玉考慮後の特攻（double）
	 *   例外　　　：無し
	 */
    public double electricBallCalc(double beforeContact,String speciesName,String itemName) {
    	if (speciesName.equals("ピカチュウ") && itemName.equals("でんきだま")) {
    		return beforeContact * 2.0;
    	} else {
    		return beforeContact;
    	}
    }

	/* 火傷計算メソッド
	 *   引数　　　：火傷考慮前の攻撃（double）、攻撃側の状態異常（String）
	 *   戻り値　　：火傷考慮後の攻撃（double）
	 *   例外　　　：無し
	 */
    public double burnCalc(double beforeAttack,String badHealth) {
    	if (badHealth.equals("burn")) {
    		return Math.floor(beforeAttack / 2.0);
    	} else {
    		return beforeAttack;
    	}
    }

	/* 自爆・大爆発計算メソッド
	 *   引数　　　：自爆・大爆発考慮前の防御（double）、防御側の自爆・大爆発選ばれたフラグ（Boolean）
	 *   戻り値　　：自爆・大爆発考慮後の防御（double）
	 *   例外　　　：無し
	 */
    public double explosionBlockHalfCalc(double beforeBlock,boolean blockHalfFlag) {
    	if (blockHalfFlag) {
    		return Math.floor(beforeBlock / 2.0);
    	} else {
    		return beforeBlock;
    	}
    }

	/* メタルパウダー計算メソッド
	 *   引数　　　：メタルパウダー考慮前の防御・特防（double）、防御側の種族名（String）、防御側の持ち物（String）
	 *   戻り値　　：メタルパウダー考慮後の防御・特防（double）
	 *   例外　　　：無し
	 */
    public double metalPowderCalc(double beforeStatus,String speciesName,String itemName) {
    	if (speciesName.equals("メタモン") && itemName.equals("メタルパウダー")) {
    		return Math.floor(beforeStatus * 1.5);
    	} else {
    		return beforeStatus;
    	}
    }

	/* リフレクター計算メソッド
	 *   引数　　　：リフレクター考慮前の防御（double）、防御側のリフレクター継続ターン（int）
	 *   戻り値　　：リフレクター考慮後の防御（double）
	 *   例外　　　：無し
	 */
    public double reflectorBlockCalc(double beforeBlock,int reflectorRestTurn) {
    	if (reflectorRestTurn > 0) {
    		return beforeBlock * 2.0;
    	} else {
    		return beforeBlock;
    	}
    }

	/* 光の壁計算メソッド
	 *   引数　　　：光の壁考慮前の特防（double）、防御側の光の壁継続ターン（int）
	 *   戻り値　　：光の壁考慮後の特防（double）
	 *   例外　　　：無し
	 */
    public double lightWallDefenceCalc(double beforeDefence,int lightWallRestTurn) {
    	if (lightWallRestTurn > 0) {
    		return beforeDefence * 2.0;
    	} else {
    		return beforeDefence;
    	}
    }

	/* プレゼントタイプ倍率計算メソッド　※v0.3版計算式で廃止
	 *   引数　　　：防御側のタイプ１（String）、防御側のタイプ２（String）
	 *   戻り値　　：10*タイプ倍率（double）
	 *   例外　　　：無し（存在しないタイプが入力された場合は10.0*2.0/3.0を返す）
	 *   参考　　　：https://seesaawiki.jp/pbs-thread/d/%a5%d7%a5%ec%a5%bc%a5%f3%a5%c8%a5%d0%a5%b0%c4%b4%ba%ba%b7%eb%b2%cc%a5%e1%a5%e2
	 */
    public double presentTypeRatioCalc(String type1,String type2) {
    	if(type1.equals("dark") && type2.equals("ice")) { return 10.0 * 4.0 / 21.0; }
    	if(type1.equals("dark") && type2.equals("flying")) { return 10.0 * 2.0 / 3.0; }
    	if(type1.equals("dark") && type2.equals("fire")) { return 10.0 * 4.0 / 3.0; }
    	if(type1.equals("dark") && type2.equals("none")) { return 10.0 * 2.0 / 3.0; }
    	if(type1.equals("rock") && type2.equals("dark")) { return 10.0 * 1.0 / 1.0; }
    	if(type1.equals("rock") && type2.equals("ground")) { return 10.0 * 1.0 / 2.0; }
    	if(type1.equals("rock") && type2.equals("flying")) { return 10.0 * 2.0 / 5.0; }
    	if(type1.equals("rock") && type2.equals("water")) { return 10.0 * 1.0 / 1.0; }
    	if(type1.equals("rock") && type2.equals("none")) { return 10.0 * 1.0 / 2.0; }
    	if(type1.equals("fighting") && type2.equals("none")) { return 10.0 * 1.0 / 3.0; }
    	if(type1.equals("grass") && type2.equals("poison")) { return 10.0 * 2.0 / 3.0; }
    	if(type1.equals("grass") && type2.equals("flying")) { return 10.0 * 2.0 / 5.0; }
    	if(type1.equals("grass") && type2.equals("psychic")) { return 10.0 * 4.0 / 3.0; }
    	if(type1.equals("grass") && type2.equals("none")) { return 10.0 * 4.0 / 3.0; }
    	if(type1.equals("ice") && type2.equals("ground")) { return 10.0 * 3.0 / 4.0; }
    	if(type1.equals("ice") && type2.equals("flying")) { return 10.0 * 1.0 / 3.0; }
    	if(type1.equals("ice") && type2.equals("psychic")) { return 10.0 * 2.0 / 3.0; }
    	if(type1.equals("ice") && type2.equals("none")) { return 10.0 * 4.0 / 3.0; }
    	if(type1.equals("ground") && type2.equals("rock")) { return 10.0 * 1.0 / 2.0; }
    	if(type1.equals("ground") && type2.equals("flying")) { return 10.0 * 2.0 / 3.0; }
    	if(type1.equals("ground") && type2.equals("none")) { return 10.0 * 8.0 / 9.0; }
    	if(type1.equals("electric") && type2.equals("steel")) { return 10.0 * 3.0 / 4.0; }
    	if(type1.equals("electric") && type2.equals("flying")) { return 10.0 * 2.0 / 3.0; }
    	if(type1.equals("electric") && type2.equals("none")) { return 10.0 * 1.0 / 2.0; }
    	if(type1.equals("poison") && type2.equals("ground")) { return 10.0 * 3.0 / 4.0; }
    	if(type1.equals("poison") && type2.equals("flying")) { return 10.0 * 2.0 / 3.0; }
    	if(type1.equals("poison") && type2.equals("none")) { return 10.0 * 2.0 / 3.0; }
    	if(type1.equals("steel") && type2.equals("ground")) { return 10.0 * 3.0 / 5.0; }
    	if(type1.equals("steel") && type2.equals("flying")) { return 10.0 * 2.0 / 5.0; }
    	if(type1.equals("steel") && type2.equals("none")) { return 10.0 * 4.0 / 21.0; }
    	if(type1.equals("flying") && type2.equals("none")) { return 10.0 * 2.0 / 3.0; }
    	if(type1.equals("fire") && type2.equals("rock")) { return 10.0 * 2.0 / 3.0; }
    	if(type1.equals("fire") && type2.equals("flying")) { return 10.0 * 2.0 / 3.0; }
    	if(type1.equals("fire") && type2.equals("none")) { return 10.0 * 4.0 / 3.0; }
    	if(type1.equals("water") && type2.equals("rock")) { return 10.0 * 2.0 / 3.0; }
    	if(type1.equals("water") && type2.equals("fighting")) { return 10.0 * 1.0 / 3.0; }
    	if(type1.equals("water") && type2.equals("ice")) { return 10.0 * 2.0 / 3.0; }
    	if(type1.equals("water") && type2.equals("ground")) { return 10.0 * 8.0 / 9.0; }
    	if(type1.equals("water") && type2.equals("electric")) { return 10.0 * 2.0 / 3.0; }
    	if(type1.equals("water") && type2.equals("poison")) { return 10.0 * 2.0 / 3.0; }
    	if(type1.equals("water") && type2.equals("flying")) { return 10.0 * 2.0 / 3.0; }
    	if(type1.equals("water") && type2.equals("psychic")) { return 10.0 * 4.0 / 3.0; }
    	if(type1.equals("water") && type2.equals("dragon")) { return 10.0 * 4.0 / 3.0; }
    	if(type1.equals("water") && type2.equals("none")) { return 10.0 * 3.0 / 4.0; }
    	if(type1.equals("bug") && type2.equals("rock")) { return 10.0 * 1.0 / 2.0; }
    	if(type1.equals("bug") && type2.equals("fighting")) { return 10.0 * 1.0 / 3.0; }
    	if(type1.equals("bug") && type2.equals("grass")) { return 10.0 * 4.0 / 3.0; }
    	if(type1.equals("bug") && type2.equals("poison")) { return 10.0 * 2.0 / 3.0; }
    	if(type1.equals("bug") && type2.equals("steel")) { return 10.0 * 2.0 / 3.0; }
    	if(type1.equals("bug") && type2.equals("flying")) { return 10.0 * 2.0 / 3.0; }
    	if(type1.equals("bug") && type2.equals("none")) { return 10.0 * 4.0 / 3.0; }
    	if(type1.equals("psychic") && type2.equals("grass")) { return 10.0 * 4.0 / 3.0; }
    	if(type1.equals("psychic") && type2.equals("flying")) { return 10.0 * 2.0 / 3.0; }
    	if(type1.equals("psychic") && type2.equals("none")) { return 10.0 * 1.0 / 1.0; }
    	if(type1.equals("ghost") && type2.equals("poison")) { return 10.0 * 2.0 / 3.0; }
    	if(type1.equals("ghost") && type2.equals("none")) { return 10.0 * 1.0 / 1.0; }
    	if(type1.equals("dragon") && type2.equals("flying")) { return 10.0 * 2.0 / 3.0; }
    	if(type1.equals("dragon") && type2.equals("none")) { return 10.0 * 4.0 / 3.0; }
    	if(type1.equals("normal") && type2.equals("flying")) { return 10.0 * 1.0 / 3.0; }
    	if(type1.equals("normal") && type2.equals("psychic")) { return 10.0 * 4.0 / 3.0; }
    	if(type1.equals("normal") && type2.equals("none")) { return 10.0 * 1.0 / 3.0; }
    	return 10.0 * 2.0 / 3.0; //例外回避（通常ここには来ない）
    }

	/* タイプ番号取得メソッド（プレゼント用）　※v0.3版計算式で廃止
	 *   引数　　　：タイプ１（String）、タイプ２（String）
	 *   戻り値　　：タイプ番号（double）
	 *   例外　　　：無し（存在しないタイプが入力された場合は0を返す）
	 */
    public double presentTypeNumberCalc(String type1,String type2) {
    	String type;
    	if (type2.equals("none")) {
    		type = type1;
    	} else {
    		type = type2;
    	}
    	if(type.equals("normal")) { return 0.0; }
    	if(type.equals("fighting")) { return 1.0; }
    	if(type.equals("flying")) { return 2.0; }
    	if(type.equals("poison")) { return 3.0; }
    	if(type.equals("ground")) { return 4.0; }
    	if(type.equals("rock")) { return 5.0; }
    	if(type.equals("bug")) { return 7.0; }
    	if(type.equals("ghost")) { return 8.0; }
    	if(type.equals("steel")) { return 9.0; }
    	if(type.equals("fire")) { return 20.0; }
    	if(type.equals("water")) { return 21.0; }
    	if(type.equals("grass")) { return 22.0; }
    	if(type.equals("electric")) { return 23.0; }
    	if(type.equals("psychic")) { return 24.0; }
    	if(type.equals("ice")) { return 25.0; }
    	if(type.equals("dragon")) { return 26.0; }
    	if(type.equals("dark")) { return 27.0; }
    	return 0.0; //例外回避（通常ここには来ない）
    }

	/* タイプ番号取得メソッド（プレゼント用）　※v0.3版
	 *   引数　　　：タイプ１（String）、タイプ２（String）
	 *   戻り値　　：タイプ番号（double）
	 *   例外　　　：無し（存在しないタイプが入力された場合は1を返す）
	 */
    public double presentTypeNumberCalc_v_0_3(String type1,String type2) {
    	String type;
    	if (type2.equals("none")) {
    		type = type1;
    	} else {
    		type = type2;
    	}
    	if(type.equals("normal")) { return 1.0; }
    	if(type.equals("fighting")) { return 1.0; }
    	if(type.equals("flying")) { return 2.0; }
    	if(type.equals("poison")) { return 3.0; }
    	if(type.equals("ground")) { return 4.0; }
    	if(type.equals("rock")) { return 5.0; }
    	if(type.equals("bug")) { return 7.0; }
    	if(type.equals("ghost")) { return 8.0; }
    	if(type.equals("steel")) { return 9.0; }
    	if(type.equals("fire")) { return 20.0; }
    	if(type.equals("water")) { return 21.0; }
    	if(type.equals("grass")) { return 22.0; }
    	if(type.equals("electric")) { return 23.0; }
    	if(type.equals("psychic")) { return 24.0; }
    	if(type.equals("ice")) { return 25.0; }
    	if(type.equals("dragon")) { return 26.0; }
    	if(type.equals("dark")) { return 27.0; }
    	return 1.0; //例外回避（通常ここには来ない）
    }

	/* ダメージ本体計算メソッド（プレゼント除く）
	 *   引数　　　：技の威力（double）、攻撃力（double）、防御力（double）、レベル（double）
	 *   戻り値　　：ダメージ本体（int）
	 *   例外　　　：無し
	 */
    public int damageBodyCalc(double movePower,double subjectValueTmp,double objectValueTmp,double levelValue) {
    	double subjectValue = subjectValueTmp;
    	double objectValue = objectValueTmp;
    	// 攻撃力または防御力が255を超える場合は4で割る
    	if (subjectValue > 255.0 || objectValue > 255.0) {
    		subjectValue = Math.floor(subjectValue / 4.0);
    		objectValue = Math.floor(objectValue / 4.0);
    	}
    	// ゼロ除算回避
    	if (objectValue == 0.0) {
    		objectValue = 1.0;
    	}
    	// 計算式
    	return (int)
    			Math.floor(Math.floor(Math.floor(levelValue*2.0/5.0+2.0)*movePower*subjectValue/objectValue)/50.0);
    }

	/* ダメージ本体計算メソッド（プレゼント用）　※v0.3版
	 *   引数　　　：技の威力（double）、攻撃力（double）、防御力（double）、レベル（double）
	 *   戻り値　　：ダメージ本体（int）
	 *   例外　　　：無し
	 */
    public int presentDamageBodyCalc_v_0_3(double movePower,double subjectValueTmp,double objectValueTmp,double levelValue) {
    	double subjectValue = subjectValueTmp;
    	double objectValue = objectValueTmp;
    	// 攻撃力または防御力が255を超える場合は4で割る
    	if (subjectValue > 255.0 || objectValue > 255.0) {
    		subjectValue = Math.floor(subjectValue / 4.0);
    		objectValue = Math.floor(objectValue / 4.0);
    	}
    	// ゼロ除算回避
    	if (objectValue == 0.0) {
    		objectValue = 1.0;
    	}
    	// 計算式
    	return (int)
    			Math.floor(Math.floor(levelValue*6.0*movePower*subjectValue/objectValue)/50.0);
    }

    /* 急所率計算メソッド
	 *   引数　　　：急所率ランク（int）、攻撃技の技コード（String）、攻撃側の種族名（String）、攻撃側の持ち物（String）
	 *   戻り値　　：急所率ランク・急所技・持ち物考慮の急所率（double）
	 *   例外　　　：無し
	 */
    public double criticalHitProbabilityCalc(int criticalHitProbabilityRank,String moveEffectCode,
    		                                   String speciesName,String itemName) {
    	double[] criticalHitProbabilityRankRatio = {6.6,12.5,25,33.3,50};
    	int criticalHitProbabilityRankIndex = criticalHitProbabilityRank;
    	if (itemName.equals("ピントレンズ")) {
    		criticalHitProbabilityRankIndex = criticalHitProbabilityRankIndex + 1;
    	}
    	if (moveEffectCode.equals("00b") || moveEffectCode.equals("27")) {
    		criticalHitProbabilityRankIndex = criticalHitProbabilityRankIndex + 2;
    	}
    	if (speciesName.equals("カモネギ") && itemName.equals("ながねぎ")) {
    		criticalHitProbabilityRankIndex = 2;
    	}
    	if (speciesName.equals("ラッキー") && itemName.equals("ラッキーパンチ")) {
    		criticalHitProbabilityRankIndex = 2;
    	}
    	if (criticalHitProbabilityRankIndex > 4) {
    		criticalHitProbabilityRankIndex = 4;
    	}
    	return criticalHitProbabilityRankRatio[criticalHitProbabilityRankIndex];
    }

	/* 急所判定メソッド
	 *   引数　　　：急所率ランク・急所技・持ち物考慮の急所率（double）
	 *   戻り値　　：命中したか否か（boolean）
	 *   例外　　　：IOException（対話式でログファイルを出力できない場合）
	 */
    public boolean criticalHitJudge(double criticalHitProbability) throws IOException {
    	Random random = new Random();
    	int criticalHitRandom = random.nextInt(10000);
    	int criticalHitBoarder = (int) Math.floor(criticalHitProbability * 100.0);
    	if (criticalHitRandom < criticalHitBoarder) {
    		DialogUtil.dialogMessageOutput("急所に当たった");
    		return true;
    	} else {
    		return false;
    	}
    }

	/* 急所ダメージ計算メソッド（プレゼント除く）
	 * 　※プレゼントの挙動を見る限り、PBSは急所貫通を別計算するのが正しい←v0.2版の話なので違うかも…問題になったら直します
	 *   引数　　　：急所考慮前のダメージ本体（int）、攻撃技のタイプ（String）、
	 *   　　　　　：攻撃側の攻撃ランクに＋６した値（0～12）（int）、攻撃側の特攻ランクに＋６した値（0～12）（int）、
	 *   　　　　　：防御側の防御ランクに＋６した値（0～12）（int）、防御側の特防ランクに＋６した値（0～12）（int）、
	 *   　　　　　：攻撃側の状態異常（String）、
	 *   　　　　　：防御側のリフレクター継続ターン（int）、防御側の光の壁継続ターン（int）
	 *   戻り値　　：急所考慮後のダメージ本体（int）
	 *   例外　　　：無し
	 */
    public int criticalHitDamageCalc(int beforeDamage,String moveType,
                                          int attackRank,int contactRank,
                                          int blockRank,int diffenseRank,
                                          String badHealth,
                                          int reflectorRestTurn,int lightWallRestTurn) {
    	int afterDamage = criticalHitDoubleCalc(beforeDamage);
    	if(moveType.equals("normal")||
    	   moveType.equals("fighting")||
    	   moveType.equals("flying")||
    	   moveType.equals("poison")||
    	   moveType.equals("ground")||
    	   moveType.equals("rock")||
    	   moveType.equals("bug")||
    	   moveType.equals("ghost")||
    	   moveType.equals("steel")||
     	   moveType.equals("physic")) {
    		if (attackRank <= blockRank) {
    			afterDamage = criticalHitRankPenetrateCalc(afterDamage,attackRank,blockRank);
    			afterDamage = criticalHitReflectorPenetrateCalc(afterDamage,reflectorRestTurn);
    			afterDamage = criticalHitBurnPenetrateCalc(afterDamage,badHealth);
    		}
    	} else {
    		if (contactRank <= diffenseRank) {
    			afterDamage = criticalHitRankPenetrateCalc(afterDamage,contactRank,diffenseRank);
    			afterDamage = criticalHitLightWallPenetrateCalc(afterDamage,lightWallRestTurn);
    		}
    	}
    	return afterDamage;
    }

	/* 急所ダメージ計算メソッド（プレゼント用）　※v0.3版で廃止
	 *   引数　　　：急所考慮前のダメージ本体（int）、プレゼントの攻撃力（double）、
	 *   　　　　　：攻撃ランクに＋６した値（0～12）（int）、
	 *   　　　　　：防御ランクに＋６した値（0～12）（int）、
	 *   　　　　　：攻撃側の状態異常（String）、
	 *   　　　　　：防御側のリフレクター継続ターン（int）
	 *   戻り値　　：急所考慮後のダメージ本体（int）
	 *   例外　　　：無し
	 *   参考　　　：https://seesaawiki.jp/pbs-thread/d/%a5%d7%a5%ec%a5%bc%a5%f3%a5%c8%a5%d0%a5%b0%c4%b4%ba%ba%b7%eb%b2%cc%a5%e1%a5%e2
	 */
    public int presentCriticalHitDamageCalc(int beforeDamage,double typeRatio,
                                          int attackRank,
                                          int blockRank,
                                          String badHealth,
                                          int reflectorRestTurn) {
    	int afterDamage = criticalHitDoubleCalc(beforeDamage);
    	double typeRatioDefault = 10.0 * 1.0; // 倍率１倍
    	if (typeRatio <= typeRatioDefault) {
    		afterDamage = criticalHitRankPenetrateCalc(afterDamage,attackRank,blockRank);
    		afterDamage = criticalHitReflectorPenetrateCalc(afterDamage,reflectorRestTurn);
    		afterDamage = criticalHitBurnPenetrateCalc(afterDamage,badHealth);
    	}
    	return afterDamage;
    }

	/* 急所時ダメージ本体２倍メソッド
	 *   引数　　　：ダメージ２倍前のダメージ本体（int）
	 *   戻り値　　：ダメージ２倍後のダメージ本体（int）
	 *   例外　　　：無し
	 */
    public int criticalHitDoubleCalc(int beforeDamage) {
    	return beforeDamage * 2;
    }

	/* 急所時ランク差貫通メソッド
	 *   引数　　　：ランク差貫通前のダメージ本体（int）、
	 *   　　　　　：攻撃・特攻ランクに＋６した値（0～12）（int）、防御・特防ランクに＋６した値（0～12）（int）
	 *   戻り値　　：ランク差貫通後のダメージ本体（int）
	 *   例外　　　：無し
	 */
    public int criticalHitRankPenetrateCalc(int beforeDamage,int subjectRank,int objectRank) {
    	double[] rankRatio = {2.0/8.0, 2.0/7.0, 2.0/6.0, 2.0/5.0, 2.0/4.0, 2.0/3.0, 2.0/2.0
                                ,3.0/2.0, 4.0/2.0, 5.0/2.0, 6.0/2.0, 7.0/2.0, 8.0/2.0};
        double subjectRatio = rankRatio[subjectRank];
        double objectRatio = rankRatio[objectRank];
    	return (int) Math.floor(beforeDamage / subjectRatio * objectRatio);
    }

	/* 急所時リフレクター貫通メソッド
	 *   引数　　　：リフレクター貫通前のダメージ本体（int）、防御側のリフレクター継続ターン（int）
	 *   戻り値　　：リフレクター貫通後のダメージ本体（int）
	 *   例外　　　：無し
	 */
    public int criticalHitReflectorPenetrateCalc(int beforeDamage,int reflectorRestTurn) {
    	if (reflectorRestTurn > 0) {
    		return beforeDamage * 2;
    	} else {
    		return beforeDamage;
    	}
    }

	/* 急所時光の壁貫通メソッド
	 *   引数　　　：光の壁貫通前のダメージ本体（int）、防御側の光の壁継続ターン（int）
	 *   戻り値　　：光の壁貫通後のダメージ本体（int）
	 *   例外　　　：無し
	 */
    public int criticalHitLightWallPenetrateCalc(int beforeDamage,int lightWallRestTurn) {
    	if (lightWallRestTurn > 0) {
    		return beforeDamage * 2;
    	} else {
    		return beforeDamage;
    	}
    }

	/* 急所時火傷貫通メソッド
	 *   引数　　　：火傷貫通前のダメージ本体（int）、攻撃側の状態異常（String）
	 *   戻り値　　：火傷貫通後のダメージ本体（int）
	 *   例外　　　：無し
	 */
    public int criticalHitBurnPenetrateCalc(int beforeDamage,String badHealth) {
    	if (badHealth.equals("burn")) {
    		return beforeDamage * 2;
    	} else {
    		return beforeDamage;
    	}
    }

	/* タイプ強化持ち物計算メソッド
	 *   引数　　　：急所考慮後のダメージ本体（int）、攻撃技のタイプ（String）、攻撃側の持ち物（String）
	 *   戻り値　　：タイプ強化持ち物考慮後のダメージ本体（int）
	 *   例外　　　：無し
	 */
    public int typeReinforceItemCalc(int beforeDamage,String moveType,String itemName) {
    	if(moveType.equals("normal") && itemName.equals("ピンクのリボン"))
    		{ return (int) Math.floor(beforeDamage * 1.1) ; }
    	if(moveType.equals("normal") && itemName.equals("みずたまリボン"))
    		{ return (int) Math.floor(beforeDamage * 1.1) ; }
    	if(moveType.equals("fighting") && itemName.equals("くろおび"))
    		{ return (int) Math.floor(beforeDamage * 1.1) ; }
    	if(moveType.equals("flying") && itemName.equals("するどいくちばし"))
    		{ return (int) Math.floor(beforeDamage * 1.1) ; }
    	if(moveType.equals("poison") && itemName.equals("どくバリ"))
    		{ return (int) Math.floor(beforeDamage * 1.1) ; }
    	if(moveType.equals("ground") && itemName.equals("やわらかいすな"))
    		{ return (int) Math.floor(beforeDamage * 1.1) ; }
    	if(moveType.equals("rock") && itemName.equals("かたいいし"))
    		{ return (int) Math.floor(beforeDamage * 1.1) ; }
    	if(moveType.equals("bug") && itemName.equals("ぎんのこな"))
    		{ return (int) Math.floor(beforeDamage * 1.1) ; }
    	if(moveType.equals("ghost") && itemName.equals("のろいのおふだ"))
    		{ return (int) Math.floor(beforeDamage * 1.1) ; }
    	if(moveType.equals("steel") && itemName.equals("メタルコート"))
    		{ return (int) Math.floor(beforeDamage * 1.1) ; }
    	if(moveType.equals("fire") && itemName.equals("もくたん"))
    		{ return (int) Math.floor(beforeDamage * 1.1) ; }
    	if(moveType.equals("water") && itemName.equals("しんぴのしずく"))
    		{ return (int) Math.floor(beforeDamage * 1.1) ; }
    	if(moveType.equals("grass") && itemName.equals("きせきのタネ"))
    		{ return (int) Math.floor(beforeDamage * 1.1) ; }
    	if(moveType.equals("electric") && itemName.equals("じしゃく"))
    		{ return (int) Math.floor(beforeDamage * 1.1) ; }
    	if(moveType.equals("psychic") && itemName.equals("まがったスプーン"))
    		{ return (int) Math.floor(beforeDamage * 1.1) ; }
    	if(moveType.equals("ice") && itemName.equals("とけないこおり"))
    		{ return (int) Math.floor(beforeDamage * 1.1) ; }
    	if(moveType.equals("dragon") && itemName.equals("りゅうのウロコ"))
    		{ return (int) Math.floor(beforeDamage * 1.1) ; }
    	if(moveType.equals("dark") && itemName.equals("くろいめがね"))
    		{ return (int) Math.floor(beforeDamage * 1.1) ; }
    	return beforeDamage;
    }

	/* 定数項＋２計算メソッド
	 *   引数　　　：タイプ強化持ち物考慮後のダメージ本体（int）
	 *   戻り値　　：定数項＋２計算後のダメージ本体（int）
	 *   例外　　　：無し
	 */
    public int constPlus2Calc(int beforeDamage) {
    	if (beforeDamage > 997) {
    		beforeDamage = 997;
    	}
    	return beforeDamage + 2;
    }

	/* 天候補正計算メソッド
	 *   引数　　　：定数項＋２計算後のダメージ本体（int）、天候名（String）、
	 *   　　　　　：攻撃技のタイプ（String）、攻撃技の技コード（String）
	 *   戻り値　　：天候補正後のダメージ本体（int）
	 *   例外　　　：無し
	 */
    public int weatherDamageCalc(int beforeDamage,String weather,String moveType,String moveEffectCode) {
    	if (weather.equals("sunny")) {
    		if (moveType.equals("fire")) {
    			return (int) Math.floor(beforeDamage * 1.5);
    		}
    		if (moveType.equals("water")) {
    			return (int) Math.floor(beforeDamage * 0.5);
    		}
    	}
    	if (weather.equals("rainy")) {
    		if (moveType.equals("fire")) {
    			return (int) Math.floor(beforeDamage * 0.5);
    		}
    		if (moveType.equals("water")) {
    			return (int) Math.floor(beforeDamage * 1.5);
    		}
    		// ソーラービーム0.5倍化
    		if (moveEffectCode.equals("97")) {
    			return (int) Math.floor(beforeDamage * 0.5);
    		}
    	}
    	return beforeDamage;
    }

	/* タイプ一致計算メソッド
	 *   引数　　　：天候補正後のダメージ本体（int）、攻撃技のタイプ（String）、
	 *   　　　　　：攻撃側のタイプ１（String）、攻撃側のタイプ２（String）
	 *   戻り値　　：タイプ一致計算後のダメージ本体（int）
	 *   例外　　　：無し
	 */
    public int typeMatchCalc(int beforeDamage,String moveType,String type1,String type2) {
    	if (moveType.equals(type1) || moveType.equals(type2)) {
    		return (int) Math.floor(beforeDamage * 1.5);
    	}
    	// 悪あがきタイプ一致バグ
    	if (moveType.equals("physic") && (type1.equals("normal") || type2.equals("normal"))) {
    		return (int) Math.floor(beforeDamage * 1.5);
    	}
    	return beforeDamage;
    }

	/* タイプ相性計算メソッド
	 *   引数　　　：タイプ一致計算後のダメージ本体（タイプ相性だけ見たい場合は100）（int）、攻撃技のタイプ（String）、
	 *   　　　　　：防御側のタイプ１（String）、防御側のタイプ２（String）、
	 *   　　　　　：攻撃側の見破るフラグ（boolean）、防御側の被見破るフラグ（boolean）
	 *   戻り値　　：タイプ相性計算後のダメージ本体（int）
	 *   例外　　　：IOException（対話式でログファイルを出力できない場合）
	 */
    public int typeCompatibilityCalc(int beforeDamage,String moveType,
    		                           String type1,String type2,
    		                           boolean seeingThroughFlag,boolean seenThroughFlag) throws IOException {
    	double typeCompatibilityRatio = 1.0;
    	String[] type = {type1,type2};
    	for(int i = 0; i < 2; i++){
        	if (type[i].equals("ghost") && seeingThroughFlag && seenThroughFlag) {
        		type[i] = "ghostST";
        	}
        	if (moveType.equals("normal")) {
        		if (type[i].equals("rock") ||
        			type[i].equals("steel")) {
        			typeCompatibilityRatio = typeCompatibilityRatio * 1.0 / 2.0;
        		}
        		else if (type[i].equals("ghost")) {
        			typeCompatibilityRatio = typeCompatibilityRatio * 0.0;
        		}
        	} else if (moveType.equals("fighting")) {
        		if (type[i].equals("normal") ||
        			type[i].equals("rock") ||
        			type[i].equals("steel") ||
        		    type[i].equals("ice") ||
        		    type[i].equals("dark")) {
        			typeCompatibilityRatio = typeCompatibilityRatio * 2.0;
        		}
        		else if (type[i].equals("flying") ||
        				type[i].equals("poison") ||
        				type[i].equals("bug") ||
            		    type[i].equals("psychic")) {
        			typeCompatibilityRatio = typeCompatibilityRatio * 1.0 / 2.0;
        		}
        		else if (type[i].equals("ghost")) {
        			typeCompatibilityRatio = typeCompatibilityRatio * 0.0;
        		}
        	} else if (moveType.equals("flying")) {
        		if (type[i].equals("fighting") ||
        			type[i].equals("bug") ||
        		    type[i].equals("grass")) {
        			typeCompatibilityRatio = typeCompatibilityRatio * 2.0;
        		}
        		else if (type[i].equals("rock") ||
        				type[i].equals("steel") ||
            		    type[i].equals("electric")) {
        			typeCompatibilityRatio = typeCompatibilityRatio * 1.0 / 2.0;
        		}
        	} else if (moveType.equals("poison")) {
        		if (type[i].equals("grass")) {
        			typeCompatibilityRatio = typeCompatibilityRatio * 2.0;
        		}
        		else if (type[i].equals("poison") ||
        				type[i].equals("ground") ||
        				type[i].equals("rock") ||
        				type[i].equals("ghost") ||
            		    type[i].equals("ghostST")) {
        			typeCompatibilityRatio = typeCompatibilityRatio * 1.0 / 2.0;
        		}
        		else if (type[i].equals("steel")) {
        			typeCompatibilityRatio = typeCompatibilityRatio * 0.0;
        		}
        	} else if (moveType.equals("ground")) {
        		if (type[i].equals("poison") ||
        			type[i].equals("rock") ||
        			type[i].equals("steel") ||
        			type[i].equals("fire") ||
        		    type[i].equals("electric")) {
        			typeCompatibilityRatio = typeCompatibilityRatio * 2.0;
        		}
        		else if (type[i].equals("bug") ||
            		    type[i].equals("grass")) {
        			typeCompatibilityRatio = typeCompatibilityRatio * 1.0 / 2.0;
        		}
        		else if (type[i].equals("flying")) {
        			typeCompatibilityRatio = typeCompatibilityRatio * 0.0;
        		}
        	} else if (moveType.equals("rock")) {
        		if (type[i].equals("flying") ||
        			type[i].equals("bug") ||
        			type[i].equals("fire") ||
        		    type[i].equals("ice")) {
        			typeCompatibilityRatio = typeCompatibilityRatio * 2.0;
        		}
        		else if (type[i].equals("fighting") ||
        				type[i].equals("ground") ||
            		    type[i].equals("steel")) {
        			typeCompatibilityRatio = typeCompatibilityRatio * 1.0 / 2.0;
        		}
        	} else if (moveType.equals("bug")) {
        		if (type[i].equals("grass") ||
        			type[i].equals("psychic") ||
        		    type[i].equals("dark")) {
        			typeCompatibilityRatio = typeCompatibilityRatio * 2.0;
        		}
        		else if (type[i].equals("fighting") ||
        				type[i].equals("flying") ||
        				type[i].equals("poison") ||
        				type[i].equals("ghost") ||
        				type[i].equals("ghostST") ||
        				type[i].equals("steel") ||
            		    type[i].equals("fire")) {
        			typeCompatibilityRatio = typeCompatibilityRatio * 1.0 / 2.0;
        		}
        	} else if (moveType.equals("ghost")) {
        		if (type[i].equals("ghost") ||
        			type[i].equals("ghostST") ||
        		    type[i].equals("psychic")) {
        			typeCompatibilityRatio = typeCompatibilityRatio * 2.0;
        		}
        		else if (type[i].equals("steel") ||
            		    type[i].equals("dark")) {
        			typeCompatibilityRatio = typeCompatibilityRatio * 1.0 / 2.0;
        		}
        		else if (type[i].equals("normal")) {
        			typeCompatibilityRatio = typeCompatibilityRatio * 0.0;
        		}
        	} else if (moveType.equals("steel")) {
        		if (type[i].equals("rock") ||
        		    type[i].equals("ice")) {
        			typeCompatibilityRatio = typeCompatibilityRatio * 2.0;
        		}
        		else if (type[i].equals("steel") ||
        				type[i].equals("fire") ||
        				type[i].equals("water") ||
            		    type[i].equals("electric")) {
        			typeCompatibilityRatio = typeCompatibilityRatio * 1.0 / 2.0;
        		}
        	} else if (moveType.equals("fire")) {
        		if (type[i].equals("bug") ||
        			type[i].equals("steel") ||
        			type[i].equals("grass") ||
        		    type[i].equals("ice")) {
        			typeCompatibilityRatio = typeCompatibilityRatio * 2.0;
        		}
        		else if (type[i].equals("rock") ||
        				type[i].equals("fire") ||
        				type[i].equals("water") ||
            		    type[i].equals("dragon")) {
        			typeCompatibilityRatio = typeCompatibilityRatio * 1.0 / 2.0;
        		}
        	} else if (moveType.equals("water")) {
        		if (type[i].equals("ground") ||
        			type[i].equals("rock") ||
        		    type[i].equals("fire")) {
        			typeCompatibilityRatio = typeCompatibilityRatio * 2.0;
        		}
        		else if (type[i].equals("water") ||
        				type[i].equals("grass") ||
            		    type[i].equals("dragon")) {
        			typeCompatibilityRatio = typeCompatibilityRatio * 1.0 / 2.0;
        		}
        	} else if (moveType.equals("grass")) {
        		if (type[i].equals("ground") ||
        			type[i].equals("rock") ||
        		    type[i].equals("water")) {
        			typeCompatibilityRatio = typeCompatibilityRatio * 2.0;
        		}
        		else if (type[i].equals("flying") ||
        				type[i].equals("poison") ||
        				type[i].equals("bug") ||
        				type[i].equals("steel") ||
        				type[i].equals("fire") ||
        				type[i].equals("grass") ||
            		    type[i].equals("dragon")) {
        			typeCompatibilityRatio = typeCompatibilityRatio * 1.0 / 2.0;
        		}
        	} else if (moveType.equals("electric")) {
        		if (type[i].equals("flying") ||
        		    type[i].equals("water")) {
        			typeCompatibilityRatio = typeCompatibilityRatio * 2.0;
        		}
        		else if (type[i].equals("grass") ||
        				type[i].equals("electric") ||
            		    type[i].equals("dragon")) {
        			typeCompatibilityRatio = typeCompatibilityRatio * 1.0 / 2.0;
        		}
        		else if (type[i].equals("ground")) {
        			typeCompatibilityRatio = typeCompatibilityRatio * 0.0;
        		}
        	} else if (moveType.equals("psychic")) {
        		if (type[i].equals("fighting") ||
        		    type[i].equals("poison")) {
        			typeCompatibilityRatio = typeCompatibilityRatio * 2.0;
        		}
        		else if (type[i].equals("steel") ||
            		    type[i].equals("psychic")) {
        			typeCompatibilityRatio = typeCompatibilityRatio * 1.0 / 2.0;
        		}
        		else if (type[i].equals("dark")) {
        			typeCompatibilityRatio = typeCompatibilityRatio * 0.0;
        		}
        	} else if (moveType.equals("ice")) {
        		if (type[i].equals("flying") ||
        			type[i].equals("ground") ||
        			type[i].equals("grass") ||
        		    type[i].equals("dragon")) {
        			typeCompatibilityRatio = typeCompatibilityRatio * 2.0;
        		}
        		else if (type[i].equals("steel") ||
        				type[i].equals("fire") ||
        				type[i].equals("water") ||
            		    type[i].equals("ice")) {
        			typeCompatibilityRatio = typeCompatibilityRatio * 1.0 / 2.0;
        		}
        	} else if (moveType.equals("dragon")) {
        		if (type[i].equals("dragon")) {
        			typeCompatibilityRatio = typeCompatibilityRatio * 2.0;
        		}
        		else if (type[i].equals("steel")) {
        			typeCompatibilityRatio = typeCompatibilityRatio * 1.0 / 2.0;
        		}
        	} else if (moveType.equals("dark")) {
        		if (type[i].equals("ghost") ||
        			type[i].equals("ghostST") ||
        		    type[i].equals("psychic")) {
        			typeCompatibilityRatio = typeCompatibilityRatio * 2.0;
        		}
        		else if (type[i].equals("fighting") ||
        				type[i].equals("steel") ||
            		    type[i].equals("dark")) {
        			typeCompatibilityRatio = typeCompatibilityRatio * 1.0 / 2.0;
        		}
        	}
    	}
    	if (typeCompatibilityRatio==0) {
        	DialogUtil.dialogMessageOutput("タイプ相性0倍で効果が無い");
    	} else if (typeCompatibilityRatio < 1.0) {
    		DialogUtil.dialogMessageOutput("タイプ相性1倍未満");
    	} else if (typeCompatibilityRatio == 1.0) {
    		DialogUtil.dialogMessageOutput("タイプ相性1倍");
    	} else {
    		DialogUtil.dialogMessageOutput("タイプ相性1倍超え");
    	}
    	return (int) Math.floor(beforeDamage * typeCompatibilityRatio);
    }

	/* ダメージN倍化計算メソッド
	 *   引数　　　：タイプ相性計算後のダメージ本体（int）、ダメージ倍化回数（int）
	 *   戻り値　　：ダメージN倍化計算後のダメージ本体（int）
	 *   例外　　　：無し
	 */
    public int damageMultipleCalc(int beforeDamage,int multipleTimes) {
    	int ratio = 1;
    	for(int i = 0; i < multipleTimes; i++){
    		ratio = ratio * 2;
    	}
    	return beforeDamage * ratio;
    }

	/* ダメージN倍化計算メソッド（PBSトリプルキックバグ再現用）
	 *   引数　　　：通常のダメージN倍化計算後のダメージ本体（int）、トリプルキック倍率（int）
	 *   戻り値　　：ダメージN倍化計算後のダメージ本体（int）
	 *   例外　　　：無し
	 */
    public int damageTripleKickBugCalc(int beforeDamage,int multipleTimes) {
    	return beforeDamage * multipleTimes;
    }

	/* ダメージランダム化計算メソッド
	 *   引数　　　：ダメージN倍化計算後のダメージ本体（int）
	 *   戻り値　　：ダメージランダム化後のダメージ本体（int）
	 *   例外　　　：IOException（対話式でログファイルを出力できない場合）
	 */
    public int damageRandomCalc(int beforeDamage) throws IOException {
    	int afterDamage;
    	Random random = new Random();
    	int damageRandom = random.nextInt(39) + 217;
    	afterDamage = (int) Math.floor(beforeDamage * damageRandom / 255);
    	if (afterDamage==0) {
        	DialogUtil.dialogMessageOutput("ダメージ0（タイプ相性が0倍ではない場合は本当に計算結果が0）");
    	}
    	return afterDamage;
    }

	/* ダメージ再ランダム化計算メソッド（プレゼントのみ）
	 *   引数　　　：ダメージランダム化後のダメージ本体（int）、プレゼント用防御力（double）
	 *   戻り値　　：ダメージ再ランダム化後のダメージ本体（int）
	 *   例外　　　：IOException（対話式でログファイルを出力できない場合）
	 *   参考　　　：https://seesaawiki.jp/pbs-thread/d/%a5%d7%a5%ec%a5%bc%a5%f3%a5%c8%a5%d0%a5%b0%c4%b4%ba%ba%b7%eb%b2%cc%a5%e1%a5%e2
	 */
    public int presentDamageReRandomCalc(int beforeDamage,double typeCodeTmp) throws IOException {
    	int typeCode;
    	int afterDamage;
    	if (typeCodeTmp == 0) {
    		typeCode = 1;
    	} else {
    		typeCode = (int) typeCodeTmp;
    	}
    	int max = (int) Math.floor(255 / typeCode);
    	int min = 217;
    	int diff = max - min;
    	if (diff < 0) {
    		diff = diff * (-1);
    	}
    	diff = diff + 1;
    	int base = Math.min(max,min);
    	Random random = new Random();
    	int damageRandom = random.nextInt(diff) + base;
    	afterDamage = (int) Math.floor(beforeDamage * damageRandom / 255);
    	if (afterDamage==0) {
        	DialogUtil.dialogMessageOutput("ダメージ0（タイプ相性が0倍ではない場合はPBSのプレゼントの予想挙動");
    	}
    	return afterDamage;
    }

	/* 気合のハチマキ発動判定メソッド
	 *   引数　　　：ダメージランダム化後のダメージ本体（int）、防御側の本体の残りHP（int）、
	 *   　　　　　：防御側の持ち物（String）
	 *   戻り値　　：気合のハチマキ発動有無（boolean）
	 *   例外　　　：無し
	 */
    public boolean spiritTriggerCalc(int damage,int actualHP,
                                        String itemName) {

    	boolean ret = false;

    	// 現HPを超えるダメージが出ている、かつまだ倒れていない、かつ気合のハチマキ所持時のみ判定
    	if (damage > actualHP && actualHP > 0 && itemName.equals("きあいのハチマキ")) {
    		Random random = new Random();
        	int spiritRandom = random.nextInt(10000);
        	int spiritBoarder = (int) Math.floor(11.7 * 100);
        	if (spiritRandom < spiritBoarder) {
        		ret = true;
        	}
    	}

    	return ret;
    }

	/* 最終ダメージ確定（現HP・HP1残し考慮）メソッド（未来予知以外）
	 *   引数　　　：ダメージランダム化後のダメージ本体（int）、防御側の本体の残りHP（int）、
	 *   　　　　　：防御側の守る成功フラグ（boolean）、防御側の直近技コード（String）、
	 *   　　　　　：攻撃側の技コード（String）、防御側の気合のハチマキ発動有無（boolean）
	 *   戻り値　　：最終ダメージ（int）
	 *   例外　　　：IOException（対話式でログファイルを出力できない場合）
	 */
    public int finalDamageCalc(int beforeDamage,int actualHP,
    		                  boolean protectSuccessFlag,String latestUsedMoveEffectCode,
    		                  String subjectMoveEffectCode,boolean spiritFlag) throws IOException {

    	int afterDamage = beforeDamage;

    	// 現HPを超える場合は現HPと同じダメージにする
    	if (afterDamage > actualHP) {
    		afterDamage = actualHP;
    	}

    	// HP1残し判定
    	if (afterDamage == actualHP && actualHP > 0) {
        	if (protectSuccessFlag && latestUsedMoveEffectCode.equals("74")) {
        		afterDamage = afterDamage - 1;
        		DialogUtil.dialogMessageOutput("堪える発動");
        	} else if (subjectMoveEffectCode.equals("65")) {
        		afterDamage = afterDamage - 1;
        		DialogUtil.dialogMessageOutput("みねうち発動");
        	} else if (spiritFlag) {
            	afterDamage = afterDamage - 1;
            	// spiritFlag = true; // 何のために入れたんだろうこの記述…意味のない記述だけど、もしかしたら気合のハチマキ発動有無を保持したかったのかもしれない…
            	DialogUtil.dialogMessageOutput("気合のハチマキ発動");
        	}
    	}

    	return afterDamage;
    }

	/* 最終ダメージ確定（現HP・HP1残し考慮）メソッド（未来予知用）
	 *   引数　　　：ダメージランダム化後のダメージ本体（int）、防御側の本体の残りHP（int）、
	 *   　　　　　：防御側の気合のハチマキ発動有無（boolean）
	 *   戻り値　　：最終ダメージ（int）
	 *   例外　　　：IOException（対話式でログファイルを出力できない場合）
	 */
    public int predictionFinalDamageCalc(int beforeDamage,int actualHP,boolean spiritFlag) throws IOException {

    	int afterDamage = beforeDamage;

    	// 現HPを超える場合は現HPと同じダメージにする
    	if (afterDamage > actualHP) {
    		afterDamage = actualHP;
    	}

    	// HP1残し判定
    	if (afterDamage == actualHP && actualHP > 0) {
        	if (spiritFlag) {
            	afterDamage = afterDamage - 1;
            	DialogUtil.dialogMessageOutput("気合のハチマキ発動");
        	}
    	}

    	return afterDamage;
    }

	/* 割合ダメージ量・割合回復量・麻痺後素早さ計算メソッド
	 *   引数　　　：ステータス（int）、割合の分子（int）、割合の分母（int）
	 *   戻り値　　：ダメージ（int）
	 *   例外　　　：無し
	 */
    public int ratioCalc(int status,int numerator,int denominator) {
    	if (denominator == 0) {
    		denominator = 1;
    	}
    	// double型にしてないのでMath.floorする意味無い…計算結果は変わらずなので放置
    	return (int) Math.floor(status * numerator / denominator);
    }

	/* 追加効果判定メソッド（命中判定、無効化判定、王者の印所持判定をしている前提）
	 *   引数　　　：追加効果発生率（百分率）（double）、メッセージ出力用追加効果名（String）
	 *   戻り値　　：追加効果発動したか否か（boolean）
	 *   例外　　　：IOException（対話式でログファイルを出力できない場合）
	 */
    public boolean additionalEffectJudge(double additionalEffectProbability,
    		                               String additionalEffectName) throws IOException {
    	Random random = new Random();
    	int additionalEffectRandom = random.nextInt(10000);
    	int additionalEffectBoarder = (int) Math.floor(additionalEffectProbability * 100.0);
    	if (additionalEffectRandom < additionalEffectBoarder) {
    		DialogUtil.dialogMessageOutput(additionalEffectName+"の追加効果発動");
    		return true;
    	} else {
    		return false;
    	}
    }

}
