package facade;

import java.io.IOException;

import calc.AfterMoveCalc;
import calc.BeforeMoveCalc;
import calc.ChangeCalc;
import calc.HandCalc;
import calc.MoveCalc;
import calc.TurnEndCalc;
import calc.TurnStartCalc;
import calc.WinningJudgeCalc;
import command.CommandRTN;
import config.Precondition;
import party.PartyInfo;
import state.PlaceState;
import state.PokeStateBaton;
import state.PokeStatePlace;
import state.PokeStateWhole;

/*
*  TurnControlFacadeクラス
*
*  ターンカット仕様等を実装し、試合全体の流れをコントロールするクラスです。
*  設計は以下のページを参照してください。
*  https://seesaawiki.jp/pbs-thread/d/%a5%bf%a1%bc%a5%f3%a5%ab%a5%c3%a5%c8%bb%c5%cd%cd
*
*  変更履歴
*  2020/11/21 完成
*  2020/11/23 規定ターンが過ぎた時に一時的に進めたターンカウントを戻さないように修正
*
*/

public class TurnControlFacade {

	/* 引数で受け取るオブジェクト */
	public PartyInfo partyInfoP1 = null;
	public PartyInfo partyInfoP2 = null;
	public PokeStatePlace pokeStatePlaceP1 = null;
	public PokeStatePlace pokeStatePlaceP2 = null;
	public PokeStateBaton pokeStateBatonP1 = null;
	public PokeStateBaton pokeStateBatonP2 = null;
	public PokeStateWhole pokeStateWholeP1 = null;
	public PokeStateWhole pokeStateWholeP2 = null;
	public PlaceState placeState = null;
	public CommandRTN commandRTN = null;
	public String partyNameP1 = "P1Party.txt";
	public String partyNameP2 = "P2Party.txt";
	public boolean partySelectOnceOnlyFlag = true; // trueなら２回目の試行以降はパーティ選択省略（処理速度向上のため）

	/* 先手・後手決定時に随時セットするオブジェクト配列（0が先手、1が後手） */
	public PartyInfo[] partyInfoHand = null;
	public PokeStatePlace[] pokeStatePlaceHand = null;
	public PokeStateBaton[] pokeStateBatonHand = null;
	public PokeStateWhole[] pokeStateWholeHand = null;
	public int nowIndex = -1;

	/* 計算用オブジェクト */
	public TurnStartCalc turnStartCalc = null;
	public HandCalc handCalc = null;
	public BeforeMoveCalc beforeMoveCalc = null;
	public MoveCalc moveCalc = null;
	public AfterMoveCalc afterMoveCalc = null;
	public ChangeCalc changeCalc = null;
	public TurnEndCalc turnEndCalc = null;
	public WinningJudgeCalc winningJudgeCalc = null;

	/* フラグ類 */
	public boolean endFlag = false; // 終了フラグ
	public boolean turnCutFlag = false; // ターンカットフラグ
	public int winner = -1; // 勝者（0なら引き分け、1ならP1勝利、2ならP2勝利）
	public int endTurn = -1; // 対戦終了時のターン数

	/* デフォルトコンストラクタ
	 *   引数　　　：無し
	 *   例外　　　：無し
	 */
    public TurnControlFacade() {

    	// オブジェクト配列生成
    	this.partyInfoHand = new PartyInfo[2];
    	this.pokeStatePlaceHand = new PokeStatePlace[2];
    	this.pokeStateBatonHand = new PokeStateBaton[2];
    	this.pokeStateWholeHand = new PokeStateWhole[2];

    	// 計算用オブジェクト生成
    	this.turnStartCalc = new TurnStartCalc();
    	this.handCalc = new HandCalc();
    	this.beforeMoveCalc = new BeforeMoveCalc();
    	this.moveCalc = new MoveCalc();
    	this.afterMoveCalc = new AfterMoveCalc();
    	this.changeCalc = new ChangeCalc();
    	this.turnEndCalc = new TurnEndCalc();
    	this.winningJudgeCalc = new WinningJudgeCalc();

    }

	/* 対戦の流れ（全体）
	 *   引数　　　：PartyInfoオブジェクト（P1）、PartyInfoオブジェクト（P2）、
	 *   　　　　　：PokeStatePlaceオブジェクト（P1）、PokeStatePlaceオブジェクト（P2）、
	 *   　　　　　：PokeStateBatonオブジェクト（P1）、PokeStatebatonオブジェクト（P2）、
	 *   　　　　　：PokeStateWholeオブジェクト（P1）、PokeStateWholeオブジェクト（P2）、
	 *   　　　　　：PlaceStateオブジェクト、
	 *   　　　　　：CommandRTNオブジェクト、
	 *   　　　　　：パーティ名（P1）（String）、パーティ名（P2）（String）、
	 *   　　　　　：試行２回目以降パーティ選択省略フラグ（boolean）
	 *   戻り値　　：勝者（int）
	 *   例外　　　：無し
	 */
    public int battleWhole(PartyInfo partyInfoP1, PartyInfo partyInfoP2,
            PokeStatePlace pokeStatePlaceP1, PokeStatePlace pokeStatePlaceP2,
            PokeStateBaton pokeStateBatonP1, PokeStateBaton pokeStateBatonP2,
            PokeStateWhole pokeStateWholeP1, PokeStateWhole pokeStateWholeP2,
            PlaceState placeState,
            CommandRTN commandRTN,
            String partyNameP1, String partyNameP2,
            boolean partySelectOnceOnlyFlag) {

    	// オブジェクト登録
    	this.partyInfoP1 = partyInfoP1;
    	this.partyInfoP2 = partyInfoP2;
    	this.pokeStatePlaceP1 = pokeStatePlaceP1;
    	this.pokeStatePlaceP2 = pokeStatePlaceP2;
    	this.pokeStateBatonP1 = pokeStateBatonP1;
    	this.pokeStateBatonP2 = pokeStateBatonP2;
    	this.pokeStateWholeP1 = pokeStateWholeP1;
    	this.pokeStateWholeP2 = pokeStateWholeP2;
    	this.placeState = placeState;
    	this.commandRTN = commandRTN;
    	this.partyNameP1 = partyNameP1;
    	this.partyNameP2 = partyNameP2;
    	this.partySelectOnceOnlyFlag = partySelectOnceOnlyFlag;

    	// 初期化（勝者リセット）
    	this.winner = -1;
    	this.endTurn = -1;

    	// 初期状態指定（パーティ選択前）
    	this.commandRTN.initStateSelectBeforePartySelect();

    	// パーティ選択（1P・2P）
    	if (this.partyInfoP1.partyTextArray[0] == null ||
    	    this.partyInfoP2.partyTextArray[0] == null ||
    	    !this.partySelectOnceOnlyFlag) {
    		this.partySelect();
    	}

    	// ポケモン選出（1P・2P）
    	this.pokemonSelect();

    	// 初期状態指定（ポケモン選出後）
    	this.commandRTN.initStateSelectAfterChoice();

    	// 終了フラグ←False
    	this.endFlag = false;

    	// 0ターン目処理（全体）
    	this.nowIndex = -1;
    	this.zeroTurnCalc();

    	// 1ターンのまとまり
    	while (!endFlag) {

    		// ターンカットフラグ←False
    		this.turnCutFlag = false;

    		// ターンカウントを1増やす　※初期値0
    		try {
				this.turnStartCalc.turnStartCalcCommon(this.pokeStatePlaceP1,
						                                this.pokeStatePlaceP2,
						                                this.placeState);
			} catch (IOException e) {
				System.out.println("ログ出力に失敗したので終了します。");
	    		System.exit(1);
			}

    		// 技選択・交代先選択（1P・2P）
    		this.commandRTN.changeStateTurnStart();
    		this.SelectMoveChange();

    		// 先手・後手判定（全体）
    		this.handCalc();

    		// 技・交代繰り出し処理（先手）
    		this.nowIndex = 0;
    		this.commandRTN.changeStateBeforeFirstHandMove();
    		this.doMoveChange();

    		// 技・交代繰り出し処理（後手）
    		this.nowIndex = 1;
    		this.commandRTN.changeStateBeforeSecondHandMove();
    		this.doMoveChange();

    		// ターン終了時処理（全体）
    		this.nowIndex = -1;
    		this.commandRTN.changeStateTurnEnd();
    		this.turnEndCalc();

    		// 規定ターン終了判定（全体）
    		this.judgeBattleEnd();

    	}

    	// 勝敗判定（全体）
    	this.winningJudge();

    	// ０ターン目開始前まで戻す初期化処理（全体）
    	this.initializeForZeroTurn();

    	// 選出前まで戻す初期化処理（全体）
    	this.initializeForBeforeChoice();

    	// パーティの初期化は不要（初期化したい場合もパーティテキストファイル読み込みで上書きされるため）

    	// 勝者を返却
    	return this.winner;

    }

    /* パーティ選択（1P・2P）
	 *   引数　　　：無し
	 *   戻り値　　：無し
	 *   例外　　　：無し
	 */
    public void partySelect () {
    	this.commandRTN.partySelect(this.partyNameP1,this.partyNameP2);
    	return;
    }

    /* ポケモン選出（1P・2P）
	 *   引数　　　：無し
	 *   戻り値　　：無し
	 *   例外　　　：無し
	 */
    public void pokemonSelect () {
    	this.commandRTN.choice();
    	this.pokeStatePlaceP1.changeBatonInitCommon(this.partyInfoP1.pokeStateUnitArray[0].speciesNameDisp);
    	this.pokeStatePlaceP2.changeBatonInitCommon(this.partyInfoP2.pokeStateUnitArray[0].speciesNameDisp);
    	this.pokeStateBatonP1.changeBatonInitCommon(this.partyInfoP1.pokeStateUnitArray[0].speciesNameDisp, false);
    	this.pokeStateBatonP2.changeBatonInitCommon(this.partyInfoP2.pokeStateUnitArray[0].speciesNameDisp, false);
    	return;
    }

    /* 0ターン目処理（全体）
	 *   引数　　　：無し
	 *   戻り値　　：無し
	 *   例外　　　：無し
	 */
    public void zeroTurnCalc () {

    	try {
    		// 0ターン目交代処理（1P）
			this.changeCalc.changeCalcBattleStart(this.partyInfoP1,this.pokeStateBatonP1);
			// 0ターン目交代処理（2P）
			this.changeCalc.changeCalcBattleStart(this.partyInfoP2,this.pokeStateBatonP2);
		} catch (IOException e) {
			System.out.println("ログ出力に失敗したので終了します。");
    		System.exit(1);
		}

    	return;

    }

    /* 技選択・交代先選択（1P・2P）
	 *   引数　　　：無し
	 *   戻り値　　：無し
	 *   例外　　　：無し
	 */
    public void SelectMoveChange () {
    	this.commandRTN.moveChange();
    	return;
    }

    /* 先手・後手判定（全体）
	 *   引数　　　：無し
	 *   戻り値　　：無し
	 *   例外　　　：無し
	 */
    public void handCalc () {

    	for (int i = 0; i < 2; i++) {
    		this.partyInfoHand[i] = null;
    		this.pokeStatePlaceHand[i] = null;
    		this.pokeStateBatonHand[i] = null;
    		this.pokeStateWholeHand[i] = null;
    	}

    	this.handCalc.handCalcCommon(this.partyInfoP1,
    			                      this.partyInfoP2,
    			                      this.pokeStatePlaceP1,
    			                      this.pokeStatePlaceP2,
    			                      this.pokeStateBatonP1,
    			                      this.pokeStateBatonP2,
    			                      this.pokeStateWholeP1,
    			                      this.pokeStateWholeP2);

		if (this.handCalc.firstHand == 1) {
			this.partyInfoHand[0] = this.partyInfoP1;
			this.partyInfoHand[1] = this.partyInfoP2;
			this.pokeStatePlaceHand[0] = this.pokeStatePlaceP1;
			this.pokeStatePlaceHand[1] = this.pokeStatePlaceP2;
			this.pokeStateBatonHand[0] = this.pokeStateBatonP1;
			this.pokeStateBatonHand[1] = this.pokeStateBatonP2;
			this.pokeStateWholeHand[0] = this.pokeStateWholeP1;
			this.pokeStateWholeHand[1] = this.pokeStateWholeP2;
			this.commandRTN.changeStateWhenHandJudge(1);
		} else {
			this.partyInfoHand[0] = this.partyInfoP2;
			this.partyInfoHand[1] = this.partyInfoP1;
			this.pokeStatePlaceHand[0] = this.pokeStatePlaceP2;
			this.pokeStatePlaceHand[1] = this.pokeStatePlaceP1;
			this.pokeStateBatonHand[0] = this.pokeStateBatonP2;
			this.pokeStateBatonHand[1] = this.pokeStateBatonP1;
			this.pokeStateWholeHand[0] = this.pokeStateWholeP2;
			this.pokeStateWholeHand[1] = this.pokeStateWholeP1;
			this.commandRTN.changeStateWhenHandJudge(2);
		}

		return;
    }

    /* 技・交代繰り出し処理（プレイヤー毎）
	 *   引数　　　：無し
	 *   戻り値　　：無し
	 *   例外　　　：無し
	 */
    public void doMoveChange () {

    	// 自分のオブジェクト配列添字取得
    	int myselfIndex = this.nowIndex;

    	// ターンカットフラグがTrueなら処理終了
    	if (this.turnCutFlag) {
    		return;
    	}

    	// 技が選択されている場合
    	if (this.pokeStatePlaceHand[myselfIndex].changeTarget == -1) {

    		// 技繰り出し判定（処理中のプレイヤー）
    		this.beforeMoveCalc();

    		// 技繰り出し処理（処理中のプレイヤー）
    		this.doMoveCalc();

    		// バトンタッチ処理（処理中のプレイヤー）
    		this.batonCalc();

    		// 吠える・吹き飛ばし処理（処理中のプレイヤー）
    		this.forcedChangeCalc();

    	// 交代が選択されている場合
    	} else {

    		// 交代処理（処理中のプレイヤー）
    		this.doChangeCalc();
    	}

    	// 死に出し判定（全体）
    	this.judgeFaintedChange();

    	// ターンカットフラグがTrueなら処理終了
    	if (this.turnCutFlag) {
    		return;
    	}

    	// 技繰り出し後処理（処理中のプレイヤー）
    	if (this.nowIndex == 0) {
    		this.commandRTN.changeStateAfterFirstHandMove();
    	} else {
    		this.commandRTN.changeStateAfterSecondHandMove();
    	}
    	this.afterMoveCalc();

    	// 死に出し判定（全体）
    	this.judgeFaintedChange();

    	return;
    }

    /* ターン終了時処理（全体）
	 *   引数　　　：無し
	 *   戻り値　　：無し
	 *   例外　　　：無し
	 */
    public void turnEndCalc () {

    	// 終了フラグがTrueなら処理終了
    	if (this.endFlag) {
    		return;
    	}

    	// ターン終了時処理（先手）
    	try {
			this.turnEndCalc.turnEndCalcCommon(this.partyInfoHand[0],
					                            this.pokeStatePlaceHand[0],
					                            this.pokeStatePlaceHand[1],
					                            this.pokeStateBatonHand[0],
					                            this.pokeStateWholeHand[0],
					                            this.placeState);
		} catch (IOException e) {
			System.out.println("ログ出力に失敗したので終了します。");
    		System.exit(1);
		}

    	// ターン終了時処理（後手）
    	try {
			this.turnEndCalc.turnEndCalcCommon(this.partyInfoHand[1],
					                            this.pokeStatePlaceHand[1],
					                            this.pokeStatePlaceHand[0],
					                            this.pokeStateBatonHand[1],
					                            this.pokeStateWholeHand[1],
					                            this.placeState);
		} catch (IOException e) {
			System.out.println("ログ出力に失敗したので終了します。");
    		System.exit(1);
		}

    	// 死に出し判定（全体）
    	this.judgeFaintedChange();

    	return;

    }

    /* 規定ターン終了判定（全体）
	 *   引数　　　：無し
	 *   戻り値　　：無し
	 *   例外　　　：無し
	 */
    public void judgeBattleEnd () {

    	// 終了フラグが立っているなら再判定しない
    	if (this.endFlag) {
    		return;
    	}
    	
    	// 終了フラグを立てるかどうか判定
    	// （一時的にPlaceStateクラスのターンカウントを進める）
    	this.placeState.turnCountUpSilent();
    	this.endFlag = this.winningJudgeCalc.winningJudgeTimingJudge(this.partyInfoP1,
                                                                      this.partyInfoP2,
                                                                      this.placeState.currentTrun,
                                                                      Precondition.endTurn);
    	
    	// 規定ターンが過ぎたならターンカウントを戻さない
    	if (this.endFlag) {
    		return;
    	}
    	
    	// 一時的に進めたターンカウントを戻す
    	this.placeState.turnCountDownSilent();

    	return;
    }

    /* 勝敗判定（全体）
	 *   引数　　　：無し
	 *   戻り値　　：無し
	 *   例外　　　：無し
	 */
    public void winningJudge () {
    	this.winningJudgeCalc.winningJudgeCommon(this.partyInfoP1,
                                                  this.partyInfoP2,
                                                  this.pokeStateBatonP1,
                                                  this.pokeStateBatonP2);
    	this.winner = this.winningJudgeCalc.winner;
    	this.endTurn = this.placeState.currentTrun;
		return;
    }

    /* ０ターン目開始前まで戻す初期化処理（全体）
	 *   引数　　　：無し
	 *   戻り値　　：無し
	 *   例外　　　：無し
	 */
    public void initializeForZeroTurn () {
    	for(int i = 0; i < 6; i++){
    		if(this.partyInfoP1.pokeStateUnitArray[i] != null) {
    			// 同一種族のポケモンがいないことが前提（２０００ルールならこの前提は必ず満たす）
    			for (int j = 0; j < 6; j++) {
    				if(this.partyInfoP1.pokeMoldArray[j] != null) {
    					if(this.partyInfoP1.pokeStateUnitArray[i].speciesName.equals(this.partyInfoP1.pokeMoldArray[j].speciesName)) {
    						this.partyInfoP1.pokeStateUnitArray[i].initialize(this.partyInfoP1.pokeMoldArray[i]);
    						j = 6;
    					}
    				}
    			}
    		}
    	}
    	for(int i = 0; i < 6; i++){
    		if(this.partyInfoP2.pokeStateUnitArray[i] != null) {
    			// 同一種族のポケモンがいないことが前提（２０００ルールならこの前提は必ず満たす）
    			for (int j = 0; j < 6; j++) {
    				if(this.partyInfoP2.pokeMoldArray[j] != null) {
    					if(this.partyInfoP2.pokeStateUnitArray[i].speciesName.equals(this.partyInfoP2.pokeMoldArray[j].speciesName)) {
    						this.partyInfoP2.pokeStateUnitArray[i].initialize(this.partyInfoP2.pokeMoldArray[i]);
    						j = 6;
    					}
    				}
    			}
    		}
    	}
    	this.pokeStatePlaceP1.changeBatonInitCommon(this.partyInfoP1.pokeStateUnitArray[0].speciesNameDisp);
    	this.pokeStatePlaceP2.changeBatonInitCommon(this.partyInfoP2.pokeStateUnitArray[0].speciesNameDisp);
    	this.pokeStateBatonP1.changeBatonInitCommon(this.partyInfoP1.pokeStateUnitArray[0].speciesNameDisp, false);
    	this.pokeStateBatonP2.changeBatonInitCommon(this.partyInfoP2.pokeStateUnitArray[0].speciesNameDisp, false);
    	this.pokeStateWholeP1.initialize();
    	this.pokeStateWholeP2.initialize();
    	this.placeState.initialize();
    	return;
    }

    /* 選出前まで戻す初期化処理（全体）
	 *   引数　　　：無し
	 *   戻り値　　：無し
	 *   例外　　　：無し
	 */
    public void initializeForBeforeChoice () {
    	this.partyInfoP1.pokeStateUnitInitialize();
    	this.partyInfoP2.pokeStateUnitInitialize();
    	this.commandRTN.changeStateInit();
    	return;
    }

    /* 相手のオブジェクト配列添字取得
	 *   引数　　　：自分のオブジェクト配列添字（int）
	 *   戻り値　　：相手のオブジェクト配列添字（int）
	 *   例外　　　：無し
	 */
    public int opponentIndexGet (int myselfIndex) {
    	return (myselfIndex + 1) % 2;
    }

    /* 技繰り出し判定（プレイヤー毎）
	 *   引数　　　：無し
	 *   戻り値　　：無し
	 *   例外　　　：無し
	 */
    public void beforeMoveCalc () {

    	// 自分・相手のオブジェクト配列添字取得
    	int myselfIndex = this.nowIndex;
    	int opponentIndex = opponentIndexGet(this.nowIndex);

    	// 本処理
    	try {
			this.beforeMoveCalc.beforeMoveCalcCommon(this.partyInfoHand[myselfIndex],
					                                  this.pokeStatePlaceHand[myselfIndex],
					                                  this.pokeStatePlaceHand[opponentIndex],
					                                  this.pokeStateBatonHand[myselfIndex],
					                                  this.pokeStateWholeHand[myselfIndex]);
		} catch (IOException e) {
			System.out.println("ログ出力に失敗したので終了します。");
    		System.exit(1);
		}

    	return;

    }

    /* 技繰り出し処理（プレイヤー毎）
	 *   引数　　　：無し
	 *   戻り値　　：無し
	 *   例外　　　：無し
	 */
    public void doMoveCalc () {

		// 自分・相手のオブジェクト配列添字取得
    	int myselfIndex = this.nowIndex;
    	int opponentIndex = opponentIndexGet(this.nowIndex);

		// 本処理
    	if (this.beforeMoveCalc.movePossibleFlag) {
			try {
				boolean firstHandFlag;
				if (myselfIndex == 0) {
					firstHandFlag = true;
				} else {
					firstHandFlag = false;
				}
				this.moveCalc.moveCalcCommon(this.partyInfoHand[myselfIndex],
						                      this.partyInfoHand[opponentIndex],
						                      this.pokeStatePlaceHand[myselfIndex],
						                      this.pokeStatePlaceHand[opponentIndex],
						                      this.pokeStateBatonHand[myselfIndex],
						                      this.pokeStateBatonHand[opponentIndex],
						                      this.pokeStateWholeHand[myselfIndex],
						                      this.pokeStateWholeHand[opponentIndex],
						                      this.placeState,
		                                      firstHandFlag);
			} catch (IOException e) {
				System.out.println("非対話式の場合は通常ここに来ることはありません");
			}
		} else {
			this.moveCalc.calcTmpOutputAreaInit();
		}

    	return;

    }

    /* バトンタッチ処理（プレイヤー毎）
	 *   引数　　　：無し
	 *   戻り値　　：無し
	 *   例外　　　：無し
	 */
    public void batonCalc () {

    	// 自分のオブジェクト配列添字取得
    	int myselfIndex = this.nowIndex;

    	// バトンが成功している場合のみ処理実行
    	if (this.pokeStatePlaceHand[myselfIndex].batonSuccessFlag) {

    		// 交代先選択（バトンタッチ交代先指定メソッド呼び出し）
    		if (this.nowIndex == 0) {
    			this.commandRTN.changeWhenBatonFirstHand();
    		} else {
    			this.commandRTN.changeWhenBatonSecondHand();
    		}

    		// 交代処理（beforeChangeTarget=-1かつmyHandFlag=trueでchangeCalcCommonメソッド呼び出し）
    		try {
    			this.changeCalc.changeCalcCommon(this.partyInfoHand[myselfIndex],
    					                          this.pokeStatePlaceHand[myselfIndex],
    					                          this.pokeStateBatonHand[myselfIndex],
    					                          this.pokeStateWholeHand[myselfIndex],
                                                  -1,
                                                  true);
    		} catch (IOException e) {
				System.out.println("ログ出力に失敗したので終了します。");
	    		System.exit(1);
			}

    		// 交代先選択状態初期化
    		this.pokeStatePlaceHand[myselfIndex].changeTargetInit();

    	}

    	return;

    }

    /* 吠える・吹き飛ばし処理（プレイヤー毎）
	 *   引数　　　：無し
	 *   戻り値　　：無し
	 *   例外　　　：無し
	 */
    public void forcedChangeCalc () {

		// 相手のオブジェクト配列添字取得　※反転することに注意
    	int opponentIndex = opponentIndexGet(this.nowIndex);

    	// 吠える・吹き飛ばしが成功している場合のみ処理実行
    	if (this.pokeStatePlaceHand[opponentIndex].forceChangedFlag) {

    		// 交代処理（beforeChangeTarget=changeTargetかつmyHandFlag=falseでchangeCalcCommonメソッド呼び出し）
    		try {
    			this.changeCalc.changeCalcCommon(this.partyInfoHand[opponentIndex],
    					                          this.pokeStatePlaceHand[opponentIndex],
    					                          this.pokeStateBatonHand[opponentIndex],
    					                          this.pokeStateWholeHand[opponentIndex],
    					                          this.pokeStatePlaceHand[opponentIndex].changeTarget,
                                                  false);
    		} catch (IOException e) {
				System.out.println("ログ出力に失敗したので終了します。");
	    		System.exit(1);
			}

    		// 交代先選択状態初期化
    		this.pokeStatePlaceHand[opponentIndex].changeTargetInit();

    	}

    	return;

    }

    /* 交代処理（プレイヤー毎）
	 *   引数　　　：無し
	 *   戻り値　　：無し
	 *   例外　　　：無し
	 */
    public void doChangeCalc () {

		// 自分のオブジェクト配列添字取得
    	int myselfIndex = this.nowIndex;

		// 本処理（beforeChangeTarget=changeTargetかつmyHandFlag=trueでchangeCalcCommonメソッド呼び出し）
    	try {
    		this.changeCalc.changeCalcCommon(this.partyInfoHand[myselfIndex],
    				                          this.pokeStatePlaceHand[myselfIndex],
    				                          this.pokeStateBatonHand[myselfIndex],
    				                          this.pokeStateWholeHand[myselfIndex],
    				                          this.pokeStatePlaceHand[myselfIndex].changeTarget,
                                              true);
		} catch (IOException e) {
			System.out.println("ログ出力に失敗したので終了します。");
    		System.exit(1);
		}

		// 交代先選択状態初期化
    	this.pokeStatePlaceHand[myselfIndex].changeTargetInit();

    	return;

    }

    /* 死に出し判定（全体）
	 *   引数　　　：無し
	 *   戻り値　　：無し
	 *   例外　　　：無し
	 */
    public void judgeFaintedChange () {

    	// どちらかのプレイヤーの場のポケモンのHPが0なら死に出し処理
    	if (this.partyInfoP1.pokeStateUnitArray[this.partyInfoP1.pokeStateUnitArrayIndex].actualHP <= 0 ||
    	    this.partyInfoP2.pokeStateUnitArray[this.partyInfoP2.pokeStateUnitArrayIndex].actualHP <= 0) {
    		this.doFaintedChange();
    	}

    	return;

    }

    /* 死に出し処理（全体）
	 *   引数　　　：無し
	 *   戻り値　　：無し
	 *   例外　　　：無し
	 */
    public void doFaintedChange () {

    	// ターンカットフラグ←True
    	this.turnCutFlag = true;

    	// どちらかのプレイヤーの残りポケモン数が0なら終了フラグを立てて終了
    	this.endFlag = this.winningJudgeCalc.winningJudgeTimingJudge(this.partyInfoP1,
                                                                      this.partyInfoP2,
                                                                      this.placeState.currentTrun,
                                                                      Precondition.endTurn);
    	if (this.endFlag) {
    		return;
    	}

    	// 交代先選択（死に出し交代先指定メソッド呼び出し）
    	// （場のポケモンのHPが0のプレイヤーについてのみ、1P・2Pで同一タイミングで選択）
    	boolean faintedFlagP1 = (this.partyInfoP1.pokeStateUnitArray[this.partyInfoP1.pokeStateUnitArrayIndex].actualHP <= 0);
    	boolean faintedFlagP2 = (this.partyInfoP2.pokeStateUnitArray[this.partyInfoP2.pokeStateUnitArrayIndex].actualHP <= 0);
    	if (faintedFlagP1) {
    		if (this.handCalc.firstHand == 1) {
    			this.commandRTN.changeWhenFaintedFirstHand();
    		} else {
    			this.commandRTN.changeWhenFaintedSecondHand();
    		}
    	}
    	if (faintedFlagP2) {
    		if (this.handCalc.firstHand == 2) {
    			this.commandRTN.changeWhenFaintedFirstHand();
    		} else {
    			this.commandRTN.changeWhenFaintedSecondHand();
    		}
    	}

    	// 交代処理（beforeChangeTarget=changeTargetかつmyHandFlag=true（何でも良い）でchangeCalcCommonメソッド呼び出し）
		// 交代先選択状態初期化
    	// （場のポケモンのHPが0のプレイヤーについてのみ、1P→2Pの順）
    	if (faintedFlagP1) {
    		try {
    			this.changeCalc.changeCalcCommon(this.partyInfoP1,
    					                          this.pokeStatePlaceP1,
    					                          this.pokeStateBatonP1,
    					                          this.pokeStateWholeP1,
    					                          this.pokeStatePlaceP1.changeTarget,
                                                  true);
    		} catch (IOException e) {
    			System.out.println("ログ出力に失敗したので終了します。");
        		System.exit(1);
    		}
    	}
    	if (faintedFlagP2) {
    		try {
    			this.changeCalc.changeCalcCommon(this.partyInfoP2,
    					                          this.pokeStatePlaceP2,
    					                          this.pokeStateBatonP2,
    					                          this.pokeStateWholeP2,
    					                          this.pokeStatePlaceP2.changeTarget,
                                                  true);
    		} catch (IOException e) {
    			System.out.println("ログ出力に失敗したので終了します。");
        		System.exit(1);
    		}
    	}

    	// 死に出し判定（全体）
    	this.judgeFaintedChange();

    	return;

    }

    /* 技繰り出し後処理（プレイヤー毎）
	 *   引数　　　：無し
	 *   戻り値　　：無し
	 *   例外　　　：無し
	 */
    public void afterMoveCalc () {

		// 自分・相手のオブジェクト配列添字取得
    	int myselfIndex = this.nowIndex;
    	int opponentIndex = opponentIndexGet(this.nowIndex);

		// 本処理
    	try {
    		this.afterMoveCalc.afterMoveCalcCommon(this.partyInfoHand[myselfIndex],
    				                                this.partyInfoHand[opponentIndex],
    				                                this.pokeStatePlaceHand[myselfIndex],
    				                                this.pokeStatePlaceHand[opponentIndex],
    				                                this.pokeStateBatonHand[myselfIndex]);
		} catch (IOException e) {
			System.out.println("ログ出力に失敗したので終了します。");
    		System.exit(1);
		}

    	return;

    }

}
