package state;

import java.io.IOException;

import util.DialogUtil;

/*
*  PlaceStateクラス
*
*  場の状態を管理するクラスです。
*  自分と相手で同一オブジェクトを共有してください。
*
*  変更履歴
*  2019/04/06 仮作成
*  2019/04/29 完成
*  2019/05/06 天候切れ判定を残りターン制から現在のターンと比較する制に変更（自分と相手の二重カウントダウン防止）
*  2019/07/13 初期化メソッドを追加
*  2019/08/13 トリプルキック倍率を追加(トリプルキックPBSバグ再現)
*  2020/11/18 ターンカウントを一時的に進める/戻すメソッドの追加
*
*/

public class PlaceState {

	/* 天候 */
    public String weather = "none"; //天候
    public int weatherEndTurn = 0; //天候切れターン（発生で現在のターン+4、現在のターンと同じになったら天候を"none"）

    /* トリプルキック倍率(PBSバグ再現) */
    public int tripleKickRatio = 1;

    /* ターン */
    public int currentTrun = 0; //現在のターン（ターン開始時に+1）

	/* 初期化メソッド
	 *   タイミング：対戦を開始する時
	 *   引数　　　：無し
	 *   戻り値　　：無し
	 *   例外　　　：無し
	 */
	public void initialize() {
		weather = "none";
		weatherEndTurn = 0;
		tripleKickRatio = 1;
		currentTrun = 0;
	}

	/* ターン終了時共通メソッド
	 *   タイミング：ターン終了時の処理が全て行われた後（双方のプレイヤーが呼び出すで問題ない）
	 *   引数　　　：無し
	 *   戻り値　　：無し
	 *   例外　　　：無し
	 */
    public void turnEndCommon() {
    	tripleKickRatio = 1;
    }

	/* 天候変更メソッド
	 *   タイミング：天候技使用時
	 *   引数　　　：技コード（にほんばれ（89） or あまごい（88） or すなあらし（73））
	 *   戻り値　　：無し
	 *   例外　　　：IOException（対話式でログファイルを出力できない場合）
	 */
    public void weatherChange(String moveEffectCode) throws IOException {
    	switch (moveEffectCode) {
    	case "89":
    		weather = "sunny";
    		weatherEndTurn = currentTrun + 4;
    		DialogUtil.dialogMessageOutput(weather+"になった");
    		break;
    	case "88":
    		weather = "rainy";
    		weatherEndTurn = currentTrun + 4;
    		DialogUtil.dialogMessageOutput(weather+"になった");
    		break;
    	case "73":
    		if (!weather.equals("dusty")) {
    			weather = "dusty";
    			weatherEndTurn = currentTrun + 4;
    			DialogUtil.dialogMessageOutput(weather+"になった");
    		} else {
    			DialogUtil.dialogMessageOutput("砂嵐のターン上書きはできない");
    		}
    		break;
    	default:break;
    	}
    }

	/* 天候カウントメソッド
	 *   タイミング：ターン終了時（優先順２（砂嵐ダメージより前））
	 *   引数　　　：無し
	 *   戻り値　　：無し
	 *   例外　　　：IOException（対話式でログファイルを出力できない場合）
	 */
    public void weatherCountDown() throws IOException {
    	if(!weather.equals("none")) {
    		if(weatherEndTurn <= currentTrun) {
    			weather = "none";
    			DialogUtil.dialogMessageOutput("通常の天候に戻った");
    		} else {
    			DialogUtil.dialogMessageOutput
    				(weather+"残り"+String.format("%01d",weatherEndTurn - currentTrun)+"ターン");
    		}
    	}
    }

	/* トリプルキック倍率セットメソッド
	 *   タイミング：トリプルキック使用時
	 *   引数　　　：トリプルキック倍率（１回目の攻撃の時に1、２回目の攻撃の時に2、３回目の攻撃の時に3をセット）
	 *   戻り値　　：無し
	 *   例外　　　：無し
	 */
    public void tripleKickRatioSet(int tripleKickRatio) {
    	this.tripleKickRatio = tripleKickRatio;
    }

	/* ターンカウントメソッド
	 *   タイミング：ターン開始時
	 *   引数　　　：無し
	 *   戻り値　　：無し
	 *   例外　　　：IOException（対話式でログファイルを出力できない場合）
	 */
    public void turnCountUp() throws IOException {
    	currentTrun++;
    	DialogUtil.dialogMessageOutput("-----------------------------------------------------------------------");
    	DialogUtil.dialogMessageOutput("現在"+String.format("%03d",currentTrun)+"ターン目");
    }

	/* ターンカウントメソッド（サイレント）
	 *   タイミング：一時的にターンを進めたい場合
	 *   引数　　　：無し
	 *   戻り値　　：無し
	 *   例外　　　：無し
	 */
    public void turnCountUpSilent() {
    	currentTrun++;
    }

	/* ターンカウント戻しメソッド（サイレント）
	 *   タイミング：一時的に進めたターンを戻したい場合
	 *   引数　　　：無し
	 *   戻り値　　：無し
	 *   例外　　　：無し
	 */
    public void turnCountDownSilent() {
    	currentTrun--;
    }

}
