package state;

import java.io.IOException;
import java.util.Random;

import util.DialogUtil;

/*
*  PokeStateBatonクラス
*
*  交代で初期化されバトンタッチで引き継がれるポケモンの状態を管理するクラスです。
*  各々のポケモン間で同一オブジェクトを共有してください。
*
*  変更履歴
*  2019/04/06 仮作成
*  2019/04/28 完成
*  2019/05/03 反則フラグの追加（残り１匹での自爆・大爆発・滅びの歌の反則負け判定で使用）
*  2019/05/05 混乱自滅フラグの追加（混乱自滅という判定結果の保持に使用）
*  2019/05/11 混乱自滅判定メソッドを追加
*  2019/06/08 混乱したポケモンへの混乱技で混乱ターンが追加されないように修正
*  2019/06/09 白い霧メソッドの誤字を訂正、宿り木をスピンで解除できることの考慮忘れを修正
*  2019/06/11 ランクコピーメソッドを追加
*  2019/06/16 通常の毒・火傷で２ターン目以降に毒々の計算になるバグを修正
*  2019/06/29 交代の共通メソッド名を変更
*  2019/07/25 滅びの歌がかかる条件の誤りを修正
*  2019/09/12 混乱自滅が初期化されないバグを修正
*  2019/11/25 気合溜めを最大４回まで積めてしまうバグを修正
*  2019/11/25 混乱自然回復のターンで混乱自滅になることがあるバグを修正
*  2020/11/23 混乱判定回数をカウントする変数を追加（ルーチン作成のため）
*  2021/01/23 ルーチンでの参照禁止変数をprivate化
*
*/

public class PokeStateBaton {

	/* 表示用項目（対話式で使用） */
	public String playerNameDisp = "Player"; //プレイヤー名
	public String speciesNameDisp = "けつばん"; //種族名

	/* 各種ランク */
	public int attackRank = 6; //攻撃ランク（処理の都合上+6した値を使用）
	public int blockRank = 6; //防御ランク（処理の都合上+6した値を使用）
	public int contactRank = 6; //特攻ランク（処理の都合上+6した値を使用）
	public int diffenceRank = 6; //特防ランク（処理の都合上+6した値を使用）
	public int speedRank = 6; //素早さランク（処理の都合上+6した値を使用）
	public int hitProbabilityRank = 6; //命中率ランク（処理の都合上+6した値を使用）
	public int avoidProbabilityRank = 6; //回避率ランク（処理の都合上+6した値を使用）
	public int criticalHitProbabilityRank = 0; //急所率ランク

	/* 身代わりHP管理 */
	public int scapegoatHP = 0; //身代わり残HP（0で身代わり無し）

	/* 各種フラグ類 */
	private int confusionRestTime = 0; //残り混乱回数（発生で2～5を入れ、混乱自滅判定前に-1）…ルーチンでの参照禁止
	public int confusionJudgeTime = -1; //混乱判定回数（非混乱時は-1、混乱で0を入れ、1回判定する度に+1）…ルーチン作成用
	public boolean confusionSelfDestructionFlag = false; //混乱自滅フラグ（混乱自滅でtrue、ターンの終わりにfalse）
	public boolean trappingFlag = false; //黒い眼差しフラグ（被黒い眼差しフラグと同時にtrue時のみ効果発揮）
	public boolean trappedFlag = false; //被黒い眼差しフラグ（黒い眼差しフラグと同時にtrue時のみ効果発揮）
	public boolean parasitedFlag = false; //宿り木フラグ
	public int morePoisonContinuousTrun = 0; //猛毒継続ターン（発生で1、ターンの終わりに+1（猛毒時、毒・火傷回復で0に））
	public boolean curcedFlag = false; //呪いフラグ
	public int beingRuinRestTrun = 0; //自分でかけた滅び残りターン数（発生で4を入れ、ターンの終わりに-1、1→0でHP0）
	public int beenRuinRestTrun = 0; //かけられた滅び残りターン数（発生で4を入れ、ターンの終わりに-1、1→0でHP0）
	public boolean whiteFogFlag = false; //白い霧フラグ
	public boolean foulFlag = false; //反則フラグ（爆破・自分の滅びの歌・道連れで倒れる時にtrue、死に出しでfalse）

	/* デフォルトコンストラクタ：何もしない
	 * 　　　　　　　　　　　　　（直接プレイヤー名を代入する場合はこちらのコンストラクタを使用）
	 */
    public PokeStateBaton() {
    }

	/* コンストラクタ：表示用項目のプレイヤー名をセット
	 * 　　　　　　　　（表示用項目のプレイヤー名を使用したい場合はこちらのコンストラクタを使用）
	 *   引数　：プレイヤー名（String）
	 */
    public PokeStateBaton(String PlayerNameDisp) {
    	this.playerNameDisp = PlayerNameDisp;
    }

	/* 交代時・バトンタッチ時・死に出し時初期化共通メソッド
	 *   タイミング：交代orバトンor死に出しを行う度
	 *   引数　　　：種族名（String）、バトンタッチか否か（boolean）
	 *   戻り値　　：無し
	 *   例外　　　：無し
	 */
    // 本当はバトンタッチなら呼び出さなくて良いようにしたかったが、メッセージ用の種族名（String）があるので…。
    public void changeBatonInitCommon(String speciesNameDisp,boolean batonFlag) {
    	this.speciesNameDisp = speciesNameDisp;
    	if(!batonFlag) {
    		attackRank = 6;
    		blockRank = 6;
    		contactRank = 6;
    		diffenceRank = 6;
    		speedRank = 6;
    		hitProbabilityRank = 6;
    		avoidProbabilityRank = 6;
    		criticalHitProbabilityRank = 0;
    		scapegoatHP = 0;
    		confusionRestTime = 0;
    		confusionJudgeTime = -1;
    		confusionSelfDestructionFlag = false;
    		trappingFlag = false;
    		trappedFlag = false;
    		parasitedFlag = false;
    		morePoisonContinuousTrun = 0;
    		curcedFlag = false;
    		beingRuinRestTrun = 0;
    		beenRuinRestTrun = 0;
    		whiteFogFlag = false;
    		foulFlag = false;
    	}
    }

	/* ターン終了時共通メソッド
	 *   タイミング：ターン終了時（各種処理が終わった後の内部的な処理）
	 *   引数　　　：毒・火傷状態ならtrueでそれ以外ならfalse（boolean）
	 *   戻り値　　：無し
	 *   例外　　　：無し
	 */
    public void turnEndCommon(boolean poisonFlag){
    	confusionSelfDestructionFlag = false;
    	if(poisonFlag && morePoisonContinuousTrun > 0) {
    		morePoisonContinuousTrun++;
    	} else {
    		morePoisonContinuousTrun = 0;
    	}
    }

	/* 攻撃ランク増減メソッド
	 *   タイミング：攻撃ランクが増減する度
	 *   引数　　　：攻撃ランク変化量（int）
	 *   戻り値　　：無し
	 *   例外　　　：IOException（対話式でログファイルを出力できない場合）
	 */
    public void attackRankUpDown(int attackRankVariation) throws IOException {
    	int attackRankDisp;
    	attackRankDisp = attackRank - 6;
    	DialogUtil.dialogMessageOutput
    		(playerNameDisp+"の"+speciesNameDisp+"の変化前攻撃ランク："+attackRankDisp);
    	attackRank = attackRank + attackRankVariation;
    	if (attackRank < 0) {
    		attackRank = 0;
    	} else if (attackRank > 12) {
    		attackRank = 12;
    	}
    	attackRankDisp = attackRank - 6;
    	DialogUtil.dialogMessageOutput
    		(playerNameDisp+"の"+speciesNameDisp+"の変化後攻撃ランク："+attackRankDisp);
    }

	/* 防御ランク増減メソッド
	 *   タイミング：防御ランクが増減する度
	 *   引数　　　：防御ランク変化量（int）
	 *   戻り値　　：無し
	 *   例外　　　：IOException（対話式でログファイルを出力できない場合）
	 */
    public void blockRankUpDown(int blockRankVariation) throws IOException {
    	int blockRankDisp;
    	blockRankDisp = blockRank - 6;
    	DialogUtil.dialogMessageOutput
    		(playerNameDisp+"の"+speciesNameDisp+"の変化前防御ランク："+blockRankDisp);
    	blockRank = blockRank + blockRankVariation;
    	if (blockRank < 0) {
    		blockRank = 0;
    	} else if (blockRank > 12) {
    		blockRank = 12;
    	}
    	blockRankDisp = blockRank - 6;
    	DialogUtil.dialogMessageOutput
    		(playerNameDisp+"の"+speciesNameDisp+"の変化後防御ランク："+blockRankDisp);
    }

	/* 特攻ランク増減メソッド
	 *   タイミング：特攻ランクが増減する度
	 *   引数　　　：特攻ランク変化量（int）
	 *   戻り値　　：無し
	 *   例外　　　：IOException（対話式でログファイルを出力できない場合）
	 */
    public void contactRankUpDown(int contactRankVariation) throws IOException {
    	int contactRankDisp;
    	contactRankDisp = contactRank - 6;
    	DialogUtil.dialogMessageOutput
    		(playerNameDisp+"の"+speciesNameDisp+"の変化前特攻ランク："+contactRankDisp);
    	contactRank = contactRank + contactRankVariation;
    	if (contactRank < 0) {
    		contactRank = 0;
    	} else if (contactRank > 12) {
    		contactRank = 12;
    	}
    	contactRankDisp = contactRank - 6;
    	DialogUtil.dialogMessageOutput
    		(playerNameDisp+"の"+speciesNameDisp+"の変化後特攻ランク："+contactRankDisp);
    }

	/* 特防ランク増減メソッド
	 *   タイミング：特防ランクが増減する度
	 *   引数　　　：特防ランク変化量（int）
	 *   戻り値　　：無し
	 *   例外　　　：IOException（対話式でログファイルを出力できない場合）
	 */
    public void diffenceRankUpDown(int diffenceRankVariation) throws IOException {
    	int diffenceRankDisp;
    	diffenceRankDisp = diffenceRank - 6;
    	DialogUtil.dialogMessageOutput
    		(playerNameDisp+"の"+speciesNameDisp+"の変化前特防ランク："+diffenceRankDisp);
    	diffenceRank = diffenceRank + diffenceRankVariation;
    	if (diffenceRank < 0) {
    		diffenceRank = 0;
    	} else if (diffenceRank > 12) {
    		diffenceRank = 12;
    	}
    	diffenceRankDisp = diffenceRank - 6;
    	DialogUtil.dialogMessageOutput
    		(playerNameDisp+"の"+speciesNameDisp+"の変化後特防ランク："+diffenceRankDisp);
    }

	/* 素早さランク増減メソッド
	 *   タイミング：素早さランクが増減する度
	 *   引数　　　：素早さランク変化量（int）
	 *   戻り値　　：無し
	 *   例外　　　：IOException（対話式でログファイルを出力できない場合）
	 */
    public void speedRankUpDown(int speedRankVariation) throws IOException {
    	int speedRankDisp;
    	speedRankDisp = speedRank - 6;
    	DialogUtil.dialogMessageOutput
    		(playerNameDisp+"の"+speciesNameDisp+"の変化前素早さランク："+speedRankDisp);
    	speedRank = speedRank + speedRankVariation;
    	if (speedRank < 0) {
    		speedRank = 0;
    	} else if (speedRank > 12) {
    		speedRank = 12;
    	}
    	speedRankDisp = speedRank - 6;
    	DialogUtil.dialogMessageOutput
    		(playerNameDisp+"の"+speciesNameDisp+"の変化後素早さランク："+speedRankDisp);
    }

	/* 命中率ランク増減メソッド
	 *   タイミング：命中率ランクが増減する度
	 *   引数　　　：命中率ランク変化量（int）
	 *   戻り値　　：無し
	 *   例外　　　：IOException（対話式でログファイルを出力できない場合）
	 */
    public void hitProbabilityRankUpDown(int hitProbabilityRankVariation) throws IOException {
    	int hitProbabilityRankDisp;
    	hitProbabilityRankDisp = hitProbabilityRank - 6;
    	DialogUtil.dialogMessageOutput
    		(playerNameDisp+"の"+speciesNameDisp+"の変化前命中率ランク："+hitProbabilityRankDisp);
    	hitProbabilityRank = hitProbabilityRank + hitProbabilityRankVariation;
    	if (hitProbabilityRank < 0) {
    		hitProbabilityRank = 0;
    	} else if (hitProbabilityRank > 12) {
    		hitProbabilityRank = 12;
    	}
    	hitProbabilityRankDisp = hitProbabilityRank - 6;
    	DialogUtil.dialogMessageOutput
    		(playerNameDisp+"の"+speciesNameDisp+"の変化後命中率ランク："+hitProbabilityRankDisp);
    }

	/* 回避率ランク増減メソッド
	 *   タイミング：回避率ランクが増減する度
	 *   引数　　　：回避率ランク変化量（int）
	 *   戻り値　　：無し
	 *   例外　　　：IOException（対話式でログファイルを出力できない場合）
	 */
    public void avoidProbabilityRankUpDown(int avoidProbabilityRankVariation) throws IOException {
    	int avoidProbabilityRankDisp;
    	avoidProbabilityRankDisp = avoidProbabilityRank - 6;
    	DialogUtil.dialogMessageOutput
    		(playerNameDisp+"の"+speciesNameDisp+"の変化前回避率ランク："+avoidProbabilityRankDisp);
    	avoidProbabilityRank = avoidProbabilityRank + avoidProbabilityRankVariation;
    	if (avoidProbabilityRank < 0) {
    		avoidProbabilityRank = 0;
    	} else if (avoidProbabilityRank > 12) {
    		avoidProbabilityRank = 12;
    	}
    	avoidProbabilityRankDisp = avoidProbabilityRank - 6;
    	DialogUtil.dialogMessageOutput
    		(playerNameDisp+"の"+speciesNameDisp+"の変化後回避率ランク："+avoidProbabilityRankDisp);
    }

	/* 急所率ランク増減メソッド
	 *   タイミング：急所率ランクが増減する度
	 *   引数　　　：急所率ランク変化量（int）
	 *   戻り値　　：無し
	 *   例外　　　：IOException（対話式でログファイルを出力できない場合）
	 */
    public void criticalHitProbabilityRankUpDown(int criticalHitProbabilityRankVariation) throws IOException {
    	int criticalHitProbabilityRankDisp;
    	criticalHitProbabilityRankDisp = criticalHitProbabilityRank;
    	DialogUtil.dialogMessageOutput
    		(playerNameDisp+"の"+speciesNameDisp+"の変化前急所率ランク："+criticalHitProbabilityRankDisp);
    	criticalHitProbabilityRank = criticalHitProbabilityRank + criticalHitProbabilityRankVariation;
    	if (criticalHitProbabilityRank < 0) {
    		criticalHitProbabilityRank = 0;
    	} else if (criticalHitProbabilityRank > 1) {
    		criticalHitProbabilityRank = 1;
    	}
    	criticalHitProbabilityRankDisp = criticalHitProbabilityRank;
    	DialogUtil.dialogMessageOutput
    		(playerNameDisp+"の"+speciesNameDisp+"の変化後急所率ランク："+criticalHitProbabilityRankDisp);
    }

	/* ランクコピーメソッド
	 *   タイミング：自己暗示、変身使用時
	 *   引数　　　：相手の攻撃ランク＋６（int）、相手の防御ランク＋６（int）、
	 *   　　　　　：相手の特攻ランク＋６（int）、相手の特防ランク＋６（int）、
	 *   　　　　　：相手の素早さランク＋６（int）、
	 *   　　　　　：相手の命中率ランク＋６（int）、相手の回避率ランク＋６（int）
	 *   戻り値　　：無し
	 *   例外　　　：IOException（対話式でログファイルを出力できない場合）
	 */
    public void rankCopy(int attackRankOpponent,int blockRankOpponent,
    		               int contactRankOpponent,int diffenceRankOpponent,
    		               int speedRankOpponent,
    		               int hitProbabilityRankOpponent,int avoidProbabilityRankOpponent) throws IOException {
    	attackRankUpDown(attackRankOpponent - attackRank);
    	blockRankUpDown(blockRankOpponent - blockRank);
    	contactRankUpDown(contactRankOpponent - contactRank);
    	diffenceRankUpDown(diffenceRankOpponent - diffenceRank);
    	speedRankUpDown(speedRankOpponent - speedRank);
    	hitProbabilityRankUpDown(hitProbabilityRankOpponent - hitProbabilityRank);
    	avoidProbabilityRankUpDown(avoidProbabilityRankOpponent - avoidProbabilityRank);
    }

	/* 身代わりHP増減メソッド
	 *   タイミング：身代わりHPが増減する度（身代わり作成時にも使用）
	 *   引数　　　：身代わりHP変化量（int）
	 *   戻り値　　：無し
	 *   例外　　　：IOException（対話式でログファイルを出力できない場合）
	 */
    public void scapegoatHPUpDown(int scapegoatHPVariation) throws IOException {
    	DialogUtil.dialogMessageOutput
    		(playerNameDisp+"の"+speciesNameDisp+"の変化前身代わりHP："+String.format("%03d",scapegoatHP));
    	scapegoatHP = scapegoatHP + scapegoatHPVariation;
    	if (scapegoatHP < 0) {
    		scapegoatHP = 0;
    	}
    	DialogUtil.dialogMessageOutput
    		(playerNameDisp+"の"+speciesNameDisp+"の変化後身代わりHP："+String.format("%03d",scapegoatHP));
    }

	/* 混乱ターン増減メソッド
	 *   タイミング：混乱発生時、混乱の自滅判定時、実で混乱回復時
	 *   引数　　　：混乱ターン変化量（int）
	 *   戻り値　　：無し
	 *   例外　　　：IOException（対話式でログファイルを出力できない場合）
	 */
    public void confusionRestTimeUpDown(int confusionRestTimeVariation) throws IOException {
    	if (confusionRestTime > 0 && confusionRestTimeVariation > 0) {
    		DialogUtil.dialogMessageOutput
				(playerNameDisp+"の"+speciesNameDisp+"は既に混乱している");
    	} else {
    		int confusionRestTimePre = confusionRestTime;
    		confusionRestTime = confusionRestTime + confusionRestTimeVariation;
    		if (confusionRestTime < 0) { // 通常通ることがない分岐だが念のため
    			confusionRestTime = 0;
    		}
    		if (confusionRestTimeVariation == -1) { // 自然カウントの場合（実で回復時にも通る可能性があるが、後で-1になるので問題無し）
    			confusionJudgeTime++;
    		}
    		if (confusionRestTimePre > 0 && confusionRestTime == 0) {
    			confusionJudgeTime = -1;
    			DialogUtil.dialogMessageOutput
    				(playerNameDisp+"の"+speciesNameDisp+"の混乱が解けた");
    		} else if(confusionRestTimePre == 0 && confusionRestTime > 0) {
    			confusionJudgeTime = 0;
    			DialogUtil.dialogMessageOutput
    				(playerNameDisp+"の"+speciesNameDisp+"は混乱になった");
    		}
    	}
    }

	/* 混乱ターン生成メソッド
	 *   タイミング：混乱発生時
	 *   引数　　　：無し
	 *   戻り値　　：混乱ターン（int）
	 *   例外　　　：無し
	 */
    public int confusionTimeGenerate(){
    	Random random = new Random();
    	return random.nextInt(4) + 2;
    }

	/* 混乱自滅判定メソッド
	 *   タイミング：技の繰り出し判定時（優先順４）
	 *   引数　　　：無し
	 *   戻り値　　：boolean（trueで混乱自滅、falseで混乱自滅しない）
	 *   例外　　　：IOException（対話式でログファイルを出力できない場合）
	 */
    public boolean confusionSelfDestructionJudge() throws IOException{
    	if (confusionRestTime > 1) {
    		Random random = new Random();
        	int selfDestructionRandom = random.nextInt(2);
        	int selfDestructionBoarder = 1;
        	if (selfDestructionRandom < selfDestructionBoarder) {
        		DialogUtil.dialogMessageOutput(playerNameDisp+"の"+speciesNameDisp+"は混乱自滅");
        		return true;
        	} else {
        		DialogUtil.dialogMessageOutput(playerNameDisp+"の"+speciesNameDisp+"は混乱しているが動ける");
        		return false;
        	}
    	} else {
    		return false;
    	}
    }

	/* 混乱中かどうかを返すメソッド
	 *   タイミング：混乱中かどうかの判定時
	 *   引数　　　：無し
	 *   戻り値　　：boolean(混乱中ならtrue、混乱してないならfalse)
	 *   例外　　　：無し
	 */
    public boolean isConfusion() {
    	return confusionRestTime > 0;
    }

	/* このターンで混乱が回復しないかどうかを返すメソッド
	 *   タイミング：このターンで混乱が回復しないかどうかの判定時
	 *   引数　　　：無し
	 *   戻り値　　：boolean(回復しないならtrue、回復するならfalse)
	 *   例外　　　：無し
	 */
    public boolean isNotConfusionRecoveryThisTrun() {
    	return confusionRestTime >= 2;
    }

	/* 混乱回復メソッド
	 *   タイミング：混乱回復時
	 *   引数　　　：無し
	 *   戻り値　　：無し
	 *   例外　　　：IOException（対話式でログファイルを出力できない場合）
	 */
    public void confusionRecovery() throws IOException{
    	confusionRestTimeUpDown(confusionRestTime * (-1));
    }

	/* 混乱自滅フラグセットメソッド
	 *   タイミング：混乱自滅時
	 *   引数　　　：無し
	 *   戻り値　　：無し
	 *   例外　　　：無し
	 */
    public void confusionSelfDestructionSet(){
    	confusionSelfDestructionFlag = true;
    }

	/* 黒い眼差しをした時用メソッド
	 *   タイミング：黒い眼差しをした時
	 *   引数　　　：無し
	 *   戻り値　　：無し
	 *   例外　　　：IOException（対話式でログファイルを出力できない場合）
	 */
    public void trapping() throws IOException{
    	trappingFlag = true;
    	DialogUtil.dialogMessageOutput(playerNameDisp+"の"+speciesNameDisp+"は黒い眼差しした");
    }

	/* 黒い眼差しをされた時用メソッド
	 *   タイミング：黒い眼差しをされた時
	 *   引数　　　：無し
	 *   戻り値　　：無し
	 *   例外　　　：IOException（対話式でログファイルを出力できない場合）
	 */
    public void trapped() throws IOException{
    	trappedFlag = true;
    	DialogUtil.dialogMessageOutput(playerNameDisp+"の"+speciesNameDisp+"は黒い眼差しされた");
    }

	/* 宿り木用メソッド
	 *   タイミング：宿り木された時、スピンした時
	 *   引数　　　：boolean（宿り木された時はtrue、スピンした時はfalse）
	 *   戻り値　　：無し
	 *   例外　　　：IOException（対話式でログファイルを出力できない場合）
	 */
    public void parasited(boolean parasitedFlag) throws IOException{
    	this.parasitedFlag = parasitedFlag;
    	if (this.parasitedFlag) {
    		DialogUtil.dialogMessageOutput(playerNameDisp+"の"+speciesNameDisp+"は宿り木された");
    	} else {
    		DialogUtil.dialogMessageOutput(playerNameDisp+"の"+speciesNameDisp+"は宿り木を解除した");
    	}
    }

	/* 猛毒ターンセットメソッド
	 *   タイミング：猛毒発生時
	 *   引数　　　：無し
	 *   戻り値　　：無し
	 *   例外　　　：IOException（対話式でログファイルを出力できない場合）
	 */
    public void morePoisonStart() throws IOException{
    	morePoisonContinuousTrun = 1;
    	DialogUtil.dialogMessageOutput(playerNameDisp+"の"+speciesNameDisp+"は猛毒になった");
    }

	/* 呪い用メソッド
	 *   タイミング：呪いされた時
	 *   引数　　　：無し
	 *   戻り値　　：無し
	 *   例外　　　：IOException（対話式でログファイルを出力できない場合）
	 */
    public void curced() throws IOException{
    	curcedFlag = true;
    	DialogUtil.dialogMessageOutput(playerNameDisp+"の"+speciesNameDisp+"は呪われた");
    }

	/* 滅びかけた時用メソッド
	 *   タイミング：滅びの歌をかけた時
	 *   引数　　　：無し
	 *   戻り値　　：無し
	 *   例外　　　：IOException（対話式でログファイルを出力できない場合）
	 */
    public void beingRuinStart() throws IOException{
    	if (beingRuinRestTrun == 0 && beenRuinRestTrun == 0) {
    		beingRuinRestTrun = 4;
    		DialogUtil.dialogMessageOutput(playerNameDisp+"の"+speciesNameDisp+"の滅びのカウントダウンが始まった");
    	}
    }

	/* 滅びかけられた時用メソッド
	 *   タイミング：滅びの歌をかけられた時
	 *   引数　　　：無し
	 *   戻り値　　：無し
	 *   例外　　　：IOException（対話式でログファイルを出力できない場合）
	 */
    public void beenRuinStart() throws IOException{
    	if (beingRuinRestTrun == 0 && beenRuinRestTrun == 0) {
    		beenRuinRestTrun = 4;
    		DialogUtil.dialogMessageOutput(playerNameDisp+"の"+speciesNameDisp+"の滅びのカウントダウンが始まった");
    	}
    }

	/* 滅びカウントダウン用メソッド
	 *   タイミング：ターン終了時（優先順４）
	 *   引数　　　：無し
	 *   戻り値　　：滅び発動有無（int、未発動なら0、発動なら1）
	 *   例外　　　：IOException（対話式でログファイルを出力できない場合）
	 */
    public int ruinCountDown() throws IOException{

    	/* 自分でかけた滅びカウント */
    	if (beingRuinRestTrun > 0) {
    		beingRuinRestTrun--;
    		if(beingRuinRestTrun == 0) {
    			foulFlag = true; //死に出しできなければ反則負け
    			DialogUtil.dialogMessageOutput(playerNameDisp+"の"+speciesNameDisp+"は滅んだ");
    			return 1;
    		} else {
    			DialogUtil.dialogMessageOutput
    				(playerNameDisp+"の"+speciesNameDisp+"の滅び残りターン："+beingRuinRestTrun);
    			return 0;
    		}

    	/* 相手にかけられた滅びカウント */
    	} else if (beenRuinRestTrun > 0) {
    		beenRuinRestTrun--;
    		if(beenRuinRestTrun == 0) {
    			DialogUtil.dialogMessageOutput(playerNameDisp+"の"+speciesNameDisp+"は滅んだ");
    			return 1;
    		} else {
    			DialogUtil.dialogMessageOutput
    				(playerNameDisp+"の"+speciesNameDisp+"の滅び残りターン："+beenRuinRestTrun);
    			return 0;
    		}

    	/* 滅び状態でなければ何もしない */
    	} else {
    		return 0;
    	}
    }

	/* 白い霧用メソッド
	 *   タイミング：白い霧した時
	 *   引数　　　：無し
	 *   戻り値　　：無し
	 *   例外　　　：IOException（対話式でログファイルを出力できない場合）
	 */
    public void beingWhiteFog() throws IOException{
    	whiteFogFlag = true;
    	DialogUtil.dialogMessageOutput(playerNameDisp+"の"+speciesNameDisp+"は白い霧に包まれた");
    }

	/* 自爆・大爆発・道連れ用メソッド
	 *   タイミング：爆破時、道連れ成功時（最後の１匹の自爆・大爆発・道連れで反則フラグを立てるためのメソッド）
	 *   引数　　　：無し
	 *   戻り値　　：無し
	 *   例外　　　：無し
	 */
    public void explosionFoulFlagSet(){
    	foulFlag = true; //この後死に出しできるなら反則フラグは解除
    }

}
