package state;

import java.io.IOException;

import util.DialogUtil;

/*
*  PokeStateWholeクラス
*
*  自分、若しくは相手の全てのポケモンに適用される状態を管理するクラスです。
*  各々のポケモン間で同一オブジェクトを共有してください。
*
*  変更履歴
*  2019/04/06 仮作成
*  2019/05/02 完成
*  2019/05/06 未来予知判定を残りターン制から現在のターンと比較する制に変更（ターン終了時に倒れた時の二重ダメージ防止）
*  2019/05/26 未来予知が命中しなかった場合のメッセージを追加
*  2019/07/13 初期化メソッドを追加
*  2020/08/15 バトン失敗時用メソッドを追加
*
*/

public class PokeStateWhole {

	/* 表示用項目（対話式で使用） */
	public String playerNameDisp = "Player"; //プレイヤー名

	/* 壁関連 */
	public int reflectorRestTurn = 0; //リフレクター残りターン（発生で5を入れ、ターンの終わりに-1）
	public int lightWallRestTurn = 0; //光の壁残りターン（発生で5を入れ、ターンの終わりに-1）
	public int mysteryProtectRestTurn = 0; //神秘の守り残りターン（発生で5を入れ、ターンの終わりに-1）

	/* 撒きびし */
	public boolean spikesFlag = false; //撒きびしフラグ

	/* 自爆・大爆発の防御半減 */
	public boolean blockHalfFlag = false; //防御半減フラグ（爆破時、ターン開始時にtrue、ターンの終わりにfalse）

	/* 未来予知 */
	public int predictionEndTrun = 0; //未来予知が来るターン（発生で現在のターン+2を入れそのターンになったらダメージ）
	public int predictionDamage = 0; //未来予知のダメージ部分保持領域（残りターン1→0時にこのダメージを相手に与える）

	/* 交代待ち（追い打ち用） */
	public boolean waitingChangeFlag = false; //交代待ちフラグ
	                                             //交代コマンド入力直後やバトンタッチ繰り出し時にtrue
	                                             //ターン終了やバトンタッチ失敗でfalse
	                                             //交代・バトンではfalseにしない（先行バトン追い打ち２倍バグ再現のため）

	/* デフォルトコンストラクタ：何もしない
	 * 　　　　　　　　　　　　　（直接プレイヤー名を代入する場合はこちらのコンストラクタを使用）
	 */
    public PokeStateWhole() {
    }

	/* 初期化メソッド
	 *   タイミング：対戦を開始する時
	 *   引数　　　：無し
	 *   戻り値　　：無し
	 *   例外　　　：無し
	 */
	public void initialize() {
		reflectorRestTurn = 0;
		lightWallRestTurn = 0;
		mysteryProtectRestTurn = 0;
		spikesFlag = false;
		blockHalfFlag = false;
		predictionEndTrun = 0;
		predictionDamage = 0;
		waitingChangeFlag = false;
	}

	/* コンストラクタ：表示用項目のプレイヤー名をセット
	 * 　　　　　　　　（表示用項目のプレイヤー名を使用したい場合はこちらのコンストラクタを使用）
	 *   引数　：プレイヤー名（String）
	 */
    public PokeStateWhole(String playerNameDisp) {
    	this.playerNameDisp = playerNameDisp;
    }

	/* ターン終了時共通メソッド
	 *   タイミング：ターン終了時の処理が全て行われた後
	 *   引数　　　：無し
	 *   戻り値　　：無し
	 *   例外　　　：無し
	 */
    public void turnEndCommon() {
    	blockHalfFlag = false;
    	waitingChangeFlag = false;
    	if (predictionEndTrun == 0) {
    		predictionDamage = 0;
    	}
    }

	/* リフレクターかけた時用メソッド
	 *   タイミング：リフレクターした時
	 *   引数　　　：無し
	 *   戻り値　　：無し
	 *   例外　　　：IOException（対話式でログファイルを出力できない場合）
	 */
    public void reflectorStart() throws IOException {
    	if (reflectorRestTurn == 0){
    		reflectorRestTurn = 5;
    		DialogUtil.dialogMessageOutput(playerNameDisp+"はリフレクターを貼った");
    	}
    }

	/* 光の壁かけた時用メソッド
	 *   タイミング：光の壁した時
	 *   引数　　　：無し
	 *   戻り値　　：無し
	 *   例外　　　：IOException（対話式でログファイルを出力できない場合）
	 */
    public void lightWallStart() throws IOException {
    	if (lightWallRestTurn == 0){
    		lightWallRestTurn = 5;
    		DialogUtil.dialogMessageOutput(playerNameDisp+"は光の壁を貼った");
    	}
    }

	/* 神秘の守りかけた時用メソッド
	 *   タイミング：神秘の守りした時
	 *   引数　　　：無し
	 *   戻り値　　：無し
	 *   例外　　　：IOException（対話式でログファイルを出力できない場合）
	 */
    public void mysteryProtectStart() throws IOException {
    	if (mysteryProtectRestTurn == 0){
    		mysteryProtectRestTurn = 5;
    		DialogUtil.dialogMessageOutput(playerNameDisp+"は神秘の守りを貼った");
    	}
    }

	/* 壁カウントダウンメソッド
	 *   タイミング：ターン終了時（優先順８）
	 *   引数　　　：無し
	 *   戻り値　　：無し
	 *   例外　　　：IOException（対話式でログファイルを出力できない場合）
	 */
    public void wallCountDown() throws IOException {

    	// リフレクター
    	if (reflectorRestTurn > 0){
    		reflectorRestTurn--;
    		if (reflectorRestTurn == 0) {
    			DialogUtil.dialogMessageOutput(playerNameDisp+"のリフレクターが切れた");
    		}
    	}

    	// 光の壁
    	if (lightWallRestTurn > 0){
    		lightWallRestTurn--;
    		if (lightWallRestTurn == 0) {
    			DialogUtil.dialogMessageOutput(playerNameDisp+"の光の壁が切れた");
    		}
    	}

    	// 神秘の守り
    	if (mysteryProtectRestTurn > 0){
    		mysteryProtectRestTurn--;
    		if (mysteryProtectRestTurn == 0) {
    			DialogUtil.dialogMessageOutput(playerNameDisp+"の神秘の守りが切れた");
    		}
    	}

    }

	/* 撒きびし制御用メソッド
	 *   タイミング：撒かれた時・スピンした時
	 *   引数　　　：boolean（撒かれた時はtrue、スピンした時はfalse）
	 *   戻り値　　：無し
	 *   例外　　　：IOException（対話式でログファイルを出力できない場合）
	 */
    public void spikesSet(boolean spikesFlag) throws IOException{
    	this.spikesFlag = spikesFlag;
    	if (spikesFlag) {
    		DialogUtil.dialogMessageOutput(playerNameDisp+"は撒きびしを撒かれた");
    	} else {
    		DialogUtil.dialogMessageOutput(playerNameDisp+"の撒きびしは解除された");
    	}
    }

	/* 自爆・大爆発選ばれた時用メソッド
	 *   タイミング：自爆・大爆発選ばれた時のコマンド入力直後
	 *   引数　　　：無し
	 *   戻り値　　：無し
	 *   例外　　　：無し
	 */
    public void explosionBlockHalf() {
    	blockHalfFlag = true;
    }

	/* 未来予知使用時用メソッド（命中判定は使用時に行い、外れならダメージ0で登録）
	 *   タイミング：未来予知を使用した時
	 *   引数　　　：ダメージ量（int）、現在のターン（int）
	 *   戻り値　　：無し
	 *   例外　　　：IOException（対話式でログファイルを出力できない場合）
	 */
    public void setPrediction(int predictionDamage,int currentTrun) throws IOException {
    	if (predictionEndTrun >= currentTrun) {
    		DialogUtil.dialogMessageOutput("未来予知の連続使用はできない");
    	} else {
    		predictionEndTrun = currentTrun + 2;
    		this.predictionDamage = predictionDamage;
    	}
    }

	/* 未来予知カウントダウン用メソッド
	 *   タイミング：ターン終了時（優先順１）
	 *   引数　　　：現在のターン（int）
	 *   戻り値　　：ダメージ量（int）
	 *   例外　　　：IOException（対話式でログファイルを出力できない場合）
	 */
    public int predictionCountDown(int currentTrun) throws IOException {
    	if (predictionEndTrun == currentTrun) {
    		predictionEndTrun = 0; // 連続で未来予知を食らうのを防ぐために初期化
    		if (predictionDamage > 0) {
    			DialogUtil.dialogMessageOutput(playerNameDisp+"は未来予知を食らった");
    		} else {
    			DialogUtil.dialogMessageOutput(playerNameDisp+"への未来予知は外れた");
    		}
    		return predictionDamage;
    	} else if (predictionEndTrun != 0) {
    		DialogUtil.dialogMessageOutput
    		(playerNameDisp+"が食らう未来予知は残り"+String.format("%01d",predictionEndTrun - currentTrun)+"ターン");
    		return 0;
    	} else {
    		return 0;
    	}
    }

	/* 交代選んだ時用メソッド
	 *   タイミング：交代コマンド入力直後やバトンタッチ繰り出し時
	 *   引数　　　：無し
	 *   戻り値　　：無し
	 *   例外　　　：無し
	 */
    public void waitingChange(){
    	waitingChangeFlag = true;
    }

	/* バトン失敗時用メソッド
	 *   タイミング：交代失敗（起こり得ない）やバトンタッチ失敗時
	 *   引数　　　：無し
	 *   戻り値　　：無し
	 *   例外　　　：無し
	 */
    public void changeFailed(){
    	waitingChangeFlag = false;
    }
}
