package util;

import java.io.BufferedWriter;
import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStreamWriter;
import java.text.SimpleDateFormat;
import java.util.Calendar;

/*
*  DialogUtilクラス
*
*  非対話式か対話式かをコントロールするスイッチ、及びメッセージ出力処理を提供するクラスです。
*  各々のクラスでは、下記の考慮を行います。
*  （メッセージ出力に関してはDialogUtilクラスのメソッドを呼び出すことで無意識的に考慮されます）
*
*  　・非対話式…自動的に結果を算出するツールを想定。
*  　　　　　　　利用者には最終的な結果のみ出せば良いのでメッセージは出力しない。
*  　　　　　　　また、プログラムで判断してコマンドを自動入力する必要がある。
*  　　　　　　　この際、不正なコマンド入力が行われた場合は、悪あがき等の無難なコマンドに置き換えることを推奨する。
*  　　　　　　　（無限ループ回避のため）
*
*  　・対話式　…都度コマンド入力するツールを想定。
*  　　　　　　　利用者に都度情報を提供する必要があるのでメッセージを出力する。
*  　　　　　　　また、標準入力等でコマンドの入力を促す必要がある。
*  　　　　　　　この際、不正なコマンド入力が行われた場合は、再入力を促すことを推奨する。
*
*  このクラスのメンバ変数を直接参照・編集したくなることはないと思うのでカプセル化しておきます。
*
*  変更履歴
*  2019/04/08 仮作成
*  2019/04/14 完成
*  2019/05/02 対話式の標準出力・ログ出力の一時中断機能を追加
*  2019/07/15 ログ出力一時中断中に終了処理を呼び出した際にログファイルを閉じるように修正
*  2020/08/22 PBSの仕様通り、1Pのみ表示・2Pのみ表示・第三者に表示でメソッドを分ける対応を実施
*
*/

public  class DialogUtil {

	/* 対話式スイッチ */
	public static int dialogSwitch = 0; //未使用だがSample1で使用しているため廃止しない
	private static int dialogSwitchPrivate = -1; //-1は未セット、0で非対話式、1で対話式、2は対話式（一時中断）

	/* ログファイル（対話式でのみ使用） */
	private static File logFile = null; //ログファイルのFileオブジェクト
	private static BufferedWriter logWR = null; //ログファイルのBufferedWriterオブジェクト

	/* 先手後手の保持 */
	private static int firstHandPlayer = 0; // 0はデフォルト、1ならP1、2ならP2

	/* 対話式スイッチのON/OFFを設定するメソッド（一度だけ実行可能）
	 *   引数　：int（0で非対話式、1で対話式）
	 *   戻り値：無し
	 *   例外　：IOException（対話式でログファイルを作成できない場合）
	 */
    public static void dialogSwitchSet(int dialogSwitchArg) throws IOException {
    	/* 既に対話式スイッチがセットされている場合は何もせず終了 */
    	if (dialogSwitchPrivate!=-1) {
    		return;
    	}

    	/* 対話式の場合 */
    	if (dialogSwitchArg==1) {
    		dialogSwitchPrivate = 1;

        	/* ログファイルパス情報 */
            Calendar calender = Calendar.getInstance();
            SimpleDateFormat sdf = new SimpleDateFormat("yyyyMMddHHmmssSSS");
            String dateTime = sdf.format(calender.getTime());
            String logFileForder="log";
            String logFilePath=logFileForder+"\\"+dateTime+".log";

            /* BufferedWriterオブジェクト生成 */
            logFile = new File(logFilePath);
            logWR = new BufferedWriter
                   (new OutputStreamWriter
                   (new FileOutputStream(logFile),"SJIS"));

        /* 非対話式の場合 */
    	} else {
    		dialogSwitchPrivate = 0;
    	}
    }

	/* 対話式かどうかを確認する処理
	 *   引数　：無し
	 *   戻り値：int（-1は未セット、0で非対話式、1で対話式、2で対話式（一時中断））
	 *   例外　：無し
	 */
    public static int getDialogSwitch() {
    	return dialogSwitchPrivate;
    }

	/* 対話式の標準出力とログを出力する処理
	 *   引数　：String（出力文字列）
	 *   戻り値：無し
	 *   例外　：IOException（対話式でログファイルを出力できない場合）
	 */
    public static void dialogMessageOutput(String message) throws IOException {
    	/* 対話式の場合 */
    	if (dialogSwitchPrivate==1) {

    		/* 標準出力 */
    		System.out.println(message);

    		/* ログ出力 */
    		if (logWR != null) {
    			logWR.write(message);
    			logWR.newLine();
    			logWR.flush();
    		}
    	}
    }

	/* 対話式の標準出力とログを出力する処理（P1用、形だけ用意）
	 *   引数　：String（出力文字列）
	 *   戻り値：無し
	 *   例外　：IOException（対話式でログファイルを出力できない場合）
	 */
    public static void dialogMessageOutputP1(String message) throws IOException {
    	dialogMessageOutput(message);
    }

	/* 対話式の標準出力とログを出力する処理（P2用、形だけ用意）
	 *   引数　：String（出力文字列）
	 *   戻り値：無し
	 *   例外　：IOException（対話式でログファイルを出力できない場合）
	 */
    public static void dialogMessageOutputP2(String message) throws IOException {
    	dialogMessageOutput(message);
    }

	/* 先手のプレイヤーの登録
	 *   引数　：int（先手のプレイヤー）
	 *   戻り値：無し
	 *   例外　：無し
	 */
    public static void firstHandRegist(int firstHandPlayerArg) {
    	firstHandPlayer = firstHandPlayerArg;
    }

	/* 対話式の標準出力とログを出力する処理（先手用、形だけ用意）
	 *   引数　：String（出力文字列）
	 *   戻り値：無し
	 *   例外　：IOException（対話式でログファイルを出力できない場合）
	 */
    public static void dialogMessageOutputFirstHand(String message) throws IOException {
    	if (firstHandPlayer==1) {
    		dialogMessageOutputP1(message);
    	} else if (firstHandPlayer==2) {
    		dialogMessageOutputP2(message);
    	}
    }

	/* 対話式の標準出力とログを出力する処理（後手用、形だけ用意）
	 *   引数　：String（出力文字列）
	 *   戻り値：無し
	 *   例外　：IOException（対話式でログファイルを出力できない場合）
	 */
    public static void dialogMessageOutputSecondHand(String message) throws IOException {
    	if (firstHandPlayer==1) {
    		dialogMessageOutputP2(message);
    	} else if (firstHandPlayer==2) {
    		dialogMessageOutputP1(message);
    	}
    }

	/* 対話式の標準出力とログ出力を一時的に中断する機能
	 *   引数　：無し
	 *   戻り値：無し
	 *   例外　：無し
	 */
    public static void dialogMessageStop() {
    	/* 対話式の場合 */
    	if (dialogSwitchPrivate==1) {
    		dialogSwitchPrivate = 2;
    	}
    }

	/* 対話式の標準出力とログ出力を再開する機能
	 *   引数　：無し
	 *   戻り値：無し
	 *   例外　：無し
	 */
    public static void dialogMessageRestart() {
    	/* 対話式（一時中断）の場合 */
    	if (dialogSwitchPrivate==2) {
    		dialogSwitchPrivate = 1;
    	}
    }

	/* 対話式の場合に使用する終了処理
	 *   引数　：無し
	 *   戻り値：無し
	 *   例外　：IOException（対話式でログファイルを閉じれない場合）
	 */
    public static void dialogEnd() throws IOException {
    	/* 対話式の場合 */
    	if (dialogSwitchPrivate==1||dialogSwitchPrivate==2) {

    		/* ログファイルを閉じる */
    		if (logWR != null) {
    			logWR.close();
    		}
    	}
    }
}
